/*********************************************************************
snort_spade.h, distributed as part of Spade v030125.1
Author: James Hoagland, Silicon Defense (hoagland@SiliconDefense.com)
copyright (c) 2002 by Silicon Defense (http://www.silicondefense.com/)
Released under GNU General Public License, see the COPYING file included
with the distribution or http://www.silicondefense.com/spice/ for details.

Please send complaints, kudos, and especially improvements and bugfixes to
hoagland@SiliconDefense.com.  As described in GNU General Public License, no
warranty is expressed for this program.
*********************************************************************/

/* Internal version control: Id: snort_spade.h,v 1.5 2002/12/19 22:37:10 jim Exp $ */

#ifndef __SNORT_SPADE_H__
#define __SNORT_SPADE_H__

/*! \file snort_spade.h
 * \brief 
 *  snort_spade.h is the header file for snort_spade.c
 */

/*! \addtogroup snort_spade
 * @{
*/

void SetupSpade();
void SpadeInit(u_char *argsstr);
void PreprocSpade(Packet *p);
void SpadeHomenetInit(u_char *args);
void SpadeDetectInit(u_char *args);
void SpadeStatInit(u_char *args);
void SpadeThreshadviseInit(u_char *args);
void SpadeAdaptInit(u_char *args);
void SpadeAdapt2Init(u_char *args);
void SpadeAdapt3Init(u_char *args);
void SpadeSurveyInit(u_char *args);
void SpadeCatchSig(int signal, void *arg);

#endif // __SNORT_SPADE_H__
/*********************************************************************
netspade.h, distributed as part of Spade v030125.1
Author: James Hoagland, Silicon Defense (hoagland@SiliconDefense.com)
copyright (c) 2002 by Silicon Defense (http://www.silicondefense.com/)
Released under GNU General Public License, see the COPYING file included
with the distribution or http://www.silicondefense.com/spice/ for details.

Please send complaints, kudos, and especially improvements and bugfixes to
hoagland@SiliconDefense.com.  As described in GNU General Public License, no
warranty is expressed for this program.
*********************************************************************/

/* Internal version control: Id: netspade.h,v 1.14 2002/12/20 06:10:03 jim Exp $ */

/*! \file netspade.h
 * \brief 
 *  netspade.h is the header file for the netspade "class"
 * \ingroup netspade_layer
 */

/*! \addtogroup netspade_layer
    @{
*/

#ifndef NETSPADE_H
#define NETSPADE_H

/*********************************************************************
netspade_features.h, distributed as part of Spade v030125.1
Author: James Hoagland, Silicon Defense (hoagland@SiliconDefense.com)
copyright (c) 2002 by Silicon Defense (http://www.silicondefense.com/)
Released under GNU General Public License, see the COPYING file included
with the distribution or http://www.silicondefense.com/spice/ for details.

netspade_features.h is contains the features definitions for netspade

Please send complaints, kudos, and especially improvements and bugfixes to
hoagland@SiliconDefense.com.  As described in GNU General Public License, no
warranty is expressed for this program.
*********************************************************************/

/* Internal version control: Id: netspade_features.h,v 1.9 2002/12/20 06:10:03 jim Exp $ */

#ifndef NETSPADE_FEATURES_H
#define NETSPADE_FEATURES_H

/*********************************************************************
spade_features.h, distributed as part of Spade v030125.1
Author: James Hoagland, Silicon Defense (hoagland@SiliconDefense.com)
copyright (c) 2002 by Silicon Defense (http://www.silicondefense.com/)
Released under GNU General Public License, see the COPYING file included
with the distribution or http://www.silicondefense.com/spice/ for details.

Please send complaints, kudos, and especially improvements and bugfixes to
hoagland@SiliconDefense.com.  As described in GNU General Public License, no
warranty is expressed for this program.
*********************************************************************/

/* Internal version control: Id: spade_features.h,v 1.9 2002/12/21 01:25:39 jim Exp $ */

#ifndef SPADE_FEATURES_H
#define SPADE_FEATURES_H

/*! \file spade_features.h
 * \brief 
 *  spade_features.h is a set of type declarations around a spade feature
 * \ingroup staterec
 */

/*! \addtogroup staterec
    @{
*/

#include <limits.h>

#define MAX_U32 0xFFFFFFFF ///< the largest number that can be represented in a u32
#define MAX_U16 0xFFFF     ///< the largest number that can be represented in a u16
#define MAX_U8  0xFF       ///< the largest number that can be represented in a u8

/* search to find a type to be u32 */
// u32 is a unsigned (exactly) 32 bit integer
#if USHRT_MAX == MAX_U32
typedef unsigned short u32;
#elif UINT_MAX == MAX_U32
typedef unsigned int u32;
#elif ULONG_MAX == MAX_U32
typedef unsigned long u32;
#else
#error could not find a 4 byte int to be u32 in types.h
#endif

/* search to find a type to be u16 */
// u16 is a unsigned (exactly) 16 bit integer
#if USHRT_MAX == MAX_U16
typedef unsigned short u16;
#elif UINT_MAX == MAX_U16
typedef unsigned int u16;
#elif ULONG_MAX == MAX_U16
typedef unsigned long u16;
#else
#error could not find a 2 byte int to be u16 in types.h
#endif

/* search to find a type to be u8 */
/* u8 is a unsigned (exactly) 8 bit integer */
#if UCHAR_MAX == MAX_U8
typedef unsigned char u8;
#elif USHRT_MAX == MAX_U8
typedef unsigned short u8;
#elif UINT_MAX == MAX_U8
typedef unsigned int u8;
#else
#error could not find a 1 byte int to be u8 in types.h
#endif


/// the maximum number of features libspade can handle
#define MAX_NUM_FEATURES 8

typedef u8 features; ///< type of the index represting the features we are storing the prob table about

/// this represents a sorted list of up to MAX_NUM_FEATURES features
typedef struct {
    u8 num;  ///< how many featrues are in the list
    features feat[MAX_NUM_FEATURES]; ///< 0-based array storing the features
} feature_list;

#endif // SPADE_FEATURES_H


/*! \file netspade_features.h
 * \ingroup netspade_layer
 * \brief 
 *  netspade_features.h is contains the features definitions for netspade
 */

/*! \addtogroup netspade_layer
    @{
*/


#define SIP             0 ///< the source IP netspade feature
#define DIP             1 ///< the dest IP netspade feature
#define SPORT           2 ///< the source port netspade feature
#define DPORT           3 ///< the dest port netspade feature
#define IPPROTO         4 ///< the IP protocol netspade feature
#define TCPFLAGS        5 ///< the TCP flags netspade feature
#define ICMPTYPE        6 ///< the ICMP type netspade feature
#define ICMPTYPECODE    7 ///< the ICMP type&code netspade feature

/// \brief the number of netspade features defined
/// \note must be no more than MAX_NUM_FEATURES in spade_features.h
//#define NETSPADE_NUM_FEATURES 6
#define NETSPADE_NUM_FEATURES 8
extern const char *featurenames[NETSPADE_NUM_FEATURES+1];

/// used for the IPPROTO file when we don't know the IP protocol
#define IPPROTO_UNKNOWN (u32)-1;

/// the value for the "origin" field in spade_event when the header was at the top level of the packet
#define PKTORIG_TOP     1
/// the value for the "origin" field in spade_event when the header was enclosed in an unreachable packet
#define PKTORIG_UNRCH   2

/*@}*/

#endif // NETSPADE_FEATURES_H

/*********************************************************************
spade_detection_types.h, distributed as part of Spade v030125.1
Author: James Hoagland, Silicon Defense (hoagland@SiliconDefense.com)
copyright (c) 2002 by Silicon Defense (http://www.silicondefense.com/)
Released under GNU General Public License, see the COPYING file included
with the distribution or http://www.silicondefense.com/spice/ for details.

Please send complaints, kudos, and especially improvements and bugfixes to
hoagland@SiliconDefense.com.  As described in GNU General Public License, no
warranty is expressed for this program.
*********************************************************************/

#ifndef SPADE_DETECTION_TYPES_H
#define SPADE_DETECTION_TYPES_H


/*! \file spade_detection_types.h
 * \ingroup netspade_layer
 * \brief 
 *  spade_detection_types.h contains constants, strings, and conversion macros
 *  for spade detection types and detector types
 */

/*! \addtogroup netspade_layer
    @{
*/


/* detector types (each spade detector is of a certain type) */
#define SPADE_DR_TYPE_UNKNOWN       0
#define SPADE_DR_TYPE_CLOSED_DPORT  1
#define SPADE_DR_TYPE_DEAD_DEST     2
#define SPADE_DR_TYPE_ODD_DPORT     3
#define SPADE_DR_TYPE_ODD_TYPECODE  4
#define SPADE_DR_TYPE_ODD_PORTDEST  5

/// map a detector type plus an relative detector number to the corresponding detector number
#define SPADE_DN_TYPE_FOR_DR_TYPE(num,index) ((num << 8) + index)

/// map detector type number to a short strings representing it
#define SPADE_DR_TYPE_SHORT4NUM(num) \
    ((num == SPADE_DR_TYPE_CLOSED_DPORT) ? "closed-dport" \
    :(num == SPADE_DR_TYPE_DEAD_DEST) ? "dead-dest" \
    :(num == SPADE_DR_TYPE_ODD_DPORT) ? "odd-dport" \
    :(num == SPADE_DR_TYPE_ODD_TYPECODE) ? "odd-typecode" \
    :(num == SPADE_DR_TYPE_ODD_PORTDEST) ? "odd-port-dest" \
    :"?" \
    )

/// map short strings representing a detector type to its number
#define SPADE_DR_TYPE_NUM4SHORT(str) \
    ((!strcmp(str,"closed-dport")) ? SPADE_DR_TYPE_CLOSED_DPORT \
    :(!strcmp(str,"dead-dest")) ? SPADE_DR_TYPE_DEAD_DEST \
    :(!strcmp(str,"odd-dport")) ? SPADE_DR_TYPE_ODD_DPORT \
    :(!strcmp(str,"odd-typecode")) ? SPADE_DR_TYPE_ODD_TYPECODE \
    :(!strcmp(str,"odd-port-dest")) ? SPADE_DR_TYPE_ODD_PORTDEST \
    :SPADE_DR_TYPE_UNKNOWN \
    )


/* detection types (a detector type can have multiple detection types) */
#define SPADE_DN_TYPE_UNKNOWN       0
#define SPADE_DN_TYPE_CLOSED_DPORT  ((SPADE_DR_TYPE_CLOSED_DPORT << 8) +0)
#define SPADE_DN_TYPE_ODD_OPEN_DPORT  ((SPADE_DR_TYPE_CLOSED_DPORT << 8) +1)
#define SPADE_DN_TYPE_ODD_DPORT  ((SPADE_DR_TYPE_CLOSED_DPORT << 8) +2)
#define SPADE_DN_TYPE_NONLIVE_DEST  ((SPADE_DR_TYPE_DEAD_DEST << 8) +0)
#define SPADE_DN_TYPE_SRC_ODD_DPORT ((SPADE_DR_TYPE_ODD_DPORT << 8) +0)
#define SPADE_DN_TYPE_ODD_TYPECODE  ((SPADE_DR_TYPE_ODD_TYPECODE << 8) +0)
#define SPADE_DN_TYPE_ODD_PORTDEST_LOWH  ((SPADE_DR_TYPE_ODD_PORTDEST << 8) +0)
//#define SPADE_DN_TYPE_ODD_PORTDEST_HIGHH  ((SPADE_DR_TYPE_ODD_PORTDEST << 8) +1)

/// map a detector type number to the detection type number
#define SPADE_DR_TYPE_FOR_DN_TYPE(num) (num >> 8)

/// obtain the default detection type number for a detector type number
#define DEFAULT_DN_TYPE_FOR_DR_TYPE(num) SPADE_DN_TYPE_FOR_DR_TYPE(num,0)

/// map detction type to very brief strings denoting the detection type
#define SPADE_DN_TYPE_BRIEF4NUM(num) \
    ((num == SPADE_DN_TYPE_CLOSED_DPORT) ? "CD" \
    :(num == SPADE_DN_TYPE_ODD_OPEN_DPORT) ? "ROD" \
    :(num == SPADE_DN_TYPE_ODD_DPORT) ? "RD" \
    :(num == SPADE_DN_TYPE_NONLIVE_DEST) ? "DD" \
    :(num == SPADE_DN_TYPE_SRC_ODD_DPORT) ? "OD" \
    :(num == SPADE_DN_TYPE_ODD_TYPECODE) ? "OT" \
    :(num == SPADE_DN_TYPE_ODD_PORTDEST_LOWH) ? "PD" \
    :"?" \
    )

/// map detction type very brief strings to the detection type number
#define SPADE_DN_TYPE_NUM4BRIEF(str) \
    ((!strcmp(str,"CD")) ? SPADE_DN_TYPE_CLOSED_DPORT \
    ((!strcmp(str,"ROD")) ? SPADE_DN_TYPE_ODD_OPEN_DPORT \
    ((!strcmp(str,"RD")) ? SPADE_DN_TYPE_ODD_DPORT \
    :(!strcmp(str,"DD")) ? SPADE_DN_TYPE_NONLIVE_DEST \
    :(!strcmp(str,"OD")) ? SPADE_DN_TYPE_SRC_ODD_DPORT \
    :(!strcmp(str,"OT")) ? SPADE_DN_TYPE_ODD_TYPECODE \
    :(!strcmp(str,"PD")) ? SPADE_DN_TYPE_ODD_PORTDEST_LOWH \
    :SPADE_DN_TYPE_UNKNOWN \
    )

/// map detction type to medium-length strings describing the detection done
#define SPADE_DN_TYPE_MEDDESCR4NUM(num) \
    ((num == SPADE_DN_TYPE_CLOSED_DPORT) ? "Closed dest port used" \
    :(num == SPADE_DN_TYPE_ODD_OPEN_DPORT) ? "Rare but open dest port used" \
    :(num == SPADE_DN_TYPE_ODD_DPORT) ? "Rare dest port used" \
    :(num == SPADE_DN_TYPE_NONLIVE_DEST) ? "Non-live dest used" \
    :(num == SPADE_DN_TYPE_SRC_ODD_DPORT) ? "Source used odd dest port" \
    :(num == SPADE_DN_TYPE_ODD_TYPECODE) ? "Odd ICMP type/code found" \
    :(num == SPADE_DN_TYPE_ODD_PORTDEST_LOWH) ? "Source used odd dest for port" \
    :"?" \
    )

/// map the medium-length strings describing the detection done to the detection type number
#define SPADE_DN_TYPE_NUM4MEDDESCR(str) \
    ((!strcmp(str,"Closed dest port used")) ? SPADE_DN_TYPE_CLOSED_DPORT \
    :(!strcmp(str,"Rare but open dest port used")) ? SPADE_DN_TYPE_ODD_OPEN_DPORT \
    :(!strcmp(str,"Rare dest port used")) ? SPADE_DN_TYPE_ODD_DPORT \
    :(!strcmp(str,"Non-live dest used")) ? SPADE_DN_TYPE_NONLIVE_DEST \
    :(!strcmp(str,"Source used odd dest port")) ? SPADE_DN_TYPE_SRC_ODD_DPORT \
    :(!strcmp(str,"Odd ICMP type/code found")) ? SPADE_DN_TYPE_ODD_TYPECODE \
    :(!strcmp(str,"Source used odd dest for port")) ? SPADE_DN_TYPE_ODD_PORTDEST_LOWH \
    :SPADE_DN_TYPE_UNKNOWN \
    )
    
/*@}*/

/* Id: spade_detection_types.h,v 1.5 2002/12/21 01:25:39 jim Exp $ */

#endif // SPADE_DETECTION_TYPES_H

/*********************************************************************
score_mgr.h, distributed as part of Spade v030125.1
Author: James Hoagland, Silicon Defense (hoagland@SiliconDefense.com)
copyright (c) 2002 by Silicon Defense (http://www.silicondefense.com/)
Released under GNU General Public License, see the COPYING file included
with the distribution or http://www.silicondefense.com/spice/ for details.

Please send complaints, kudos, and especially improvements and bugfixes to
hoagland@SiliconDefense.com.  As described in GNU General Public License, no
warranty is expressed for this program.
*********************************************************************/

/* Internal version control: Id: score_mgr.h,v 1.8 2002/12/20 06:10:03 jim Exp $ */

#ifndef SCORE_MGR_H
#define SCORE_MGR_H

/*! \file score_mgr.h
 * \brief 
 *  score_mgr.h is the header file for score_mgr.c
 * \ingroup stmgr
 */

/*! \weakgroup stmgr
    @{
*/

#include <stdio.h>

/*********************************************************************
thresh_adviser.h, distributed as part of Spade v030125.1
Author: James Hoagland, Silicon Defense (hoagland@SiliconDefense.com)
copyright (c) 2002 by Silicon Defense (http://www.silicondefense.com/)
Released under GNU General Public License, see the COPYING file included
with the distribution or http://www.silicondefense.com/spice/ for details.

Please send complaints, kudos, and especially improvements and bugfixes to
hoagland@SiliconDefense.com.  As described in GNU General Public License, no
warranty is expressed for this program.
*********************************************************************/

/* Internal version control: Id: thresh_adviser.h,v 1.7 2003/01/14 17:45:31 jim Exp $ */

#ifndef THRESH_ADVISER_H
#define THRESH_ADVISER_H

/*! \file thresh_adviser.h
 * \brief 
 *  thresh_adviser.h is the header file for thresh_adviser.c
 * \ingroup stmgr
 */

/*! \weakgroup stmgr
    @{
*/

#include <stdio.h>
/*********************************************************************
spade_enviro.h, distributed as part of Spade v030125.1
Author: James Hoagland, Silicon Defense (hoagland@SiliconDefense.com)
copyright (c) 2002 by Silicon Defense (http://www.silicondefense.com/)
Released under GNU General Public License, see the COPYING file included
with the distribution or http://www.silicondefense.com/spice/ for details.

Please send complaints, kudos, and especially improvements and bugfixes to
hoagland@SiliconDefense.com.  As described in GNU General Public License, no
warranty is expressed for this program.
*********************************************************************/

/* Internal version control: Id: spade_enviro.h,v 1.8 2002/12/21 01:25:39 jim Exp $ */

#ifndef SPADE_ENVIRO_H
#define SPADE_ENVIRO_H

/*! \file spade_enviro.h
 * \brief 
 *  spade_enviro.h contains the type declaration for the spade_enviro
 *  struct
 * \ingroup stmgr
 */

/*! \weakgroup stmgr
    @{
*/


#include <time.h>

/// various counts of the disposition of events; yes, this should be in netspade, not libspade because it violates abstractions
typedef struct {
    int scored;      ///< count of all packets scored 
    int respchecked; ///< count of all packets checked as a response
    int reported;    ///< count of all packets reported
    int excluded;    ///< count of all packets not reported due to configured exclusion
    int nonexcluded; ///< count of all packets that were anomalous but not excluded
    int waited;      ///< count of all packets added to the wait queue
    int insuffobsed; ///< count of packets with insufficient obsercations
} spade_pkt_stats;


/// shared state between libspade and its user
typedef struct {
    time_t now; ///< the time of the last packet added

    /// the threshold at which anomolous events are reported
    double thresh;
    
    unsigned long *total_pkts; ///< pointer to read-only count of all packets seen
    spade_pkt_stats pkt_stats; ///< packet disposition counts
} spade_enviro;

spade_enviro *new_spade_enviro(double thresh,unsigned long *total_pkts_ptr);
void init_spade_enviro(spade_enviro *self,double thresh,unsigned long *total_pkts_ptr);

/*@}*/
#endif // SPADE_ENVIRO_H

/*********************************************************************
ll_double.h, distributed as part of Spade v030125.1
Author: James Hoagland, Silicon Defense (hoagland@SiliconDefense.com)
copyright (c) 2002 by Silicon Defense (http://www.silicondefense.com/)
Released under GNU General Public License, see the COPYING file included
with the distribution or http://www.silicondefense.com/spice/ for details.

Please send complaints, kudos, and especially improvements and bugfixes to
hoagland@SiliconDefense.com.  As described in GNU General Public License, no
warranty is expressed for this program.
*********************************************************************/

#ifndef LL_DOUBLE_H
#define LL_DOUBLE_H

/*! \file ll_double.h
 * \brief
 *  ll_double.h is the header file for ll_double.c
 * \ingroup libspade_util
 */

/*! \addtogroup libspade_util
    @{
*/

/// a link in a singly linked list of doubles
typedef struct _ll_double {
    double val; ///< the value
    struct _ll_double *next; ///< the next in the list
} ll_double;

ll_double *new_ll_double(double val);
void free_ll_double_list(ll_double *start);

/*@}*/
#endif  /* ! LL_DOUBLE_H */

/* Id: ll_double.h,v 1.5 2002/12/20 06:10:03 jim Exp $ */

/*********************************************************************
spade_output.h, distributed as part of Spade v030125.1
Author: James Hoagland, Silicon Defense (hoagland@SiliconDefense.com)
copyright (c) 2002 by Silicon Defense (http://www.silicondefense.com/)
Released under GNU General Public License, see the COPYING file included
with the distribution or http://www.silicondefense.com/spice/ for details.

Please send complaints, kudos, and especially improvements and bugfixes to
hoagland@SiliconDefense.com.  As described in GNU General Public License, no
warranty is expressed for this program.
*********************************************************************/

/* Internal version control: Id: spade_output.h,v 1.3 2002/12/21 01:25:39 jim Exp $ */

#ifndef SPADE_OUTPUT_H
#define SPADE_OUTPUT_H

/*! \file spade_output.h
 * \brief 
 *  spade_output.h contains some declarations regarding Spade
 *  message output
 * \ingroup libspade_misc
 */

/** \addtogroup libspade_misc
    @{
*/

#include <stdarg.h>

/// the max length of a message from Spade
#define MAX_SPADE_MSG_LEN 1000

/// enum listing the different messages types from Spade to the user
typedef enum {
    SPADE_MSG_TYPE_STATUS,  ///< message is providing routine status information, e.g., from starting up
    SPADE_MSG_TYPE_INFO,    ///< the message is merely informational
    SPADE_MSG_TYPE_DEBUG,   ///< it is a debugging message
    SPADE_MSG_TYPE_WARNING, ///< the message is a warning
    SPADE_MSG_TYPE_FATAL    ///< fatal error
} spade_message_type;

/// function type for a spade message callback function
typedef void (*spade_msg_fn)(spade_message_type msg_type,const char *str);

void default_spade_msg_fn(spade_message_type msg_type,const char *str);

void formatted_spade_msg_send(spade_message_type msg_type,spade_msg_fn msg_fn,const char *format,...);

/*@}*/

#endif // SPADE_OUTPUT_H


/// representation of a threshold adviser
typedef struct {
    int obs_size;  ///< the number of anomalous packets desired
    time_t obs_secs; ///< how long to observe for
    /// head of a linked list of the highest anomaly scores we've seen
    /** this list can be up to tl_obs_size+1 long and is ordered by increasing score; the list is initialized to 0 -> 0 in case we never see enough packets */
    ll_double *top_anom_list; 
    int top_anom_list_size; ///< the number of scores on the list (0-based)
    time_t obs_start_time; ///< the start time of the observation, set after the first packet we see
} thresh_adviser;


void init_thresh_adviser(thresh_adviser *self, int obs_size, int obs_secs, spade_msg_fn msg_callback);
void init_thresh_adviser_from_str(thresh_adviser *self, char *str, spade_msg_fn msg_callback);
thresh_adviser *new_thresh_adviser(int obs_size, int obs_secs, spade_msg_fn msg_callback);
void thresh_adviser_reset(thresh_adviser *self, int obs_size, int obs_secs, spade_msg_fn msg_callback);
void thresh_adviser_start_time(thresh_adviser *self, time_t time);
int thresh_adviser_new_time(thresh_adviser *self, spade_enviro *enviro);
void thresh_adviser_new_score(thresh_adviser *self, double anom_score);
void thresh_adviser_write_advice(thresh_adviser *self, FILE *file);

void thresh_adviser_print_config_details(thresh_adviser *self, FILE *f, char *indent);

/*@}*/
#endif // THRESH_ADVISER_H

/*********************************************************************
thresh_adapter.h, distributed as part of Spade v030125.1
Author: James Hoagland, Silicon Defense (hoagland@SiliconDefense.com)
copyright (c) 2002 by Silicon Defense (http://www.silicondefense.com/)
Released under GNU General Public License, see the COPYING file included
with the distribution or http://www.silicondefense.com/spice/ for details.

thresh_adapter.h is the header file for thresh_adapter.c

Please send complaints, kudos, and especially improvements and bugfixes to
hoagland@SiliconDefense.com.  As described in GNU General Public License, no
warranty is expressed for this program.
*********************************************************************/

/* Internal version control: Id: thresh_adapter.h,v 1.7 2003/01/14 17:45:31 jim Exp $ */

#ifndef THRESH_ADAPTER_H
#define THRESH_ADAPTER_H

/*! \file thresh_adapter.h
 * \brief 
 *  thresh_adapter.h is the header file for thresh_adapter.c
 * \ingroup stmgr
 */

/*! \weakgroup stmgr
    @{
*/

/*********************************************************************
dll_double.h, distributed as part of Spade v030125.1
Author: James Hoagland, Silicon Defense (hoagland@SiliconDefense.com)
copyright (c) 2002 by Silicon Defense (http://www.silicondefense.com/)
Released under GNU General Public License, see the COPYING file included
with the distribution or http://www.silicondefense.com/spice/ for details.

Please send complaints, kudos, and especially improvements and bugfixes to
hoagland@SiliconDefense.com.  As described in GNU General Public License, no
warranty is expressed for this program.
*********************************************************************/

#ifndef DLL_DOUBLE_H
#define DLL_DOUBLE_H

/*! \file dll_double.h
 * \brief 
 *  dll_double.h is the header file for dll_double.c.
 * \ingroup libspade_util
 */

/*! \addtogroup libspade_util
    @{
*/

/// a link in a doubly linked list of doubles
typedef struct _dll_double {
    double val; ///< the value
    struct _dll_double *prev; ///< the previous item on the linked list
    struct _dll_double *next; ///< the next item on the linked list
} dll_double;

dll_double *new_dll_double(double val);
void free_dll_double_list(dll_double *start);


#endif  /* ! DLL_DOUBLE_H */


/* Id: dll_double.h,v 1.4 2002/12/19 22:37:10 jim Exp $ */
 
 
 

#include <stdio.h>
#include <time.h>

/// structure to hold the data for adapt mode #1
typedef struct {
    /// the number of alerts that is ideal for the given length of time
    int target;
    /// the length of time in which to ideally produce the given number of alerts; also the interval at which to adjust the report threshold
    time_t period;
    /// the weight to give to the new observation ideal cutoff in determining the new weight
    float new_obs_weight;
    /// adapt by count or by time only
    int by_count;
    
    /// the head of the list of anomaly scores.
    ll_double *top_list;
    /// the current size of this list (0-based)
    int top_list_size;
} adapt1_data;

/// structure to hold the data for adapt mode #2
typedef struct {
    /// the specification of the target
    double targetspec;
    /// the observation period
    double obsper;
    /// the number of short periods in a medium period
    int NS;
    /// the number of medium periods in a long period
    int NM;
    /// the number of long periods to average over
    int NL;
    
    /// the current target based on targetspec
    int target;
    /// latest medium term component
    double mid_anom_comp;
    /// latest long term component
    double long_anom_comp;
    /// an array of heads of observation linked lists
    dll_double **obslists_head;
    /// an array of tails of the observation linked lists
    dll_double **obslists_tail;
    /// an array of the (0-based) size of these lists
    int *obslists_size;
    /// the number of complete observation periods
    int obsper_count;
    /// arrays of short and medium term components used for calculating other components
    double *recScomps,*recMcomps;

    /// the start time of the current observation period
    time_t obsper_start;
    /// which obslist to add to, aka, obsper_count % NS
    int obslist_new_slot;

    // the count of period 2 instances
    int per2_count;
    // the count of period 3 instances
    int per3_count; 
} adapt2_data;

/// structure to hold the data for adapt mode #3
typedef struct {
    /// the specification of the target
    double targetspec;
    /// the observation period
    double obsper;
    /// the number of observation period results to average together
    int NO;
    
    /// the current target based on targetspec
    int target;
    /// array of past observations
    double *hist;
    /// a linked list of anomaly scores from the current period
    ll_double *anoms;
    /// (0-based) size of this list
    int anoms_size;
    /// number of completed observation periods
    int completed_obs_per;
    
    double obssum; ///< the sum of all current elements in the array
} adapt3_data;
 
/// structure to hold the data for adapt mode #4
typedef struct {
    double thresh; ///< the threshold to change to
} adapt4_data;

/// the representation of a threshold adapter
typedef struct {
    /// the adapt mode number
    int adapt_mode;
    /// the adapt-type specific storage; the union is selected by adapt_mode
    union {
        adapt1_data a1;
        adapt2_data a2;
        adapt3_data a3;
        adapt4_data a4;
    } d;
    
    /// adapt by count or by time only
    int adapt_by_count;
    
    /// the total count of packets at the start of the observation period
    int last_total_stats;
    /// how often (in secs) to recalc packet rate and to trigger adapting
    time_t adapt_period; 
    /// how many packet periods have occurred
    int pkt_period_count;
    /// the time this period started
    time_t period_start;
    /// the rate of all packets in the adapt period; used to decide when to adapt
    float period_pkt_rate;
    /// rate of accepted packets in the adapt period; sometimes used to determine target rate of reports
    float period_acc_rate; 
    /// are we done adapting?
    int done;
    /// the function to call to provide a message to the user
    spade_msg_fn msg_callback;
} thresh_adapter;


void init_thresh_adapter(thresh_adapter *self,spade_msg_fn msg_callback);
thresh_adapter *new_thresh_adapter(spade_msg_fn msg_callback);
void thresh_adapter_setup_from_str(thresh_adapter *self,int adaptmode,char *str);
void thresh_adapter_setup_1(thresh_adapter *self, int target, time_t period, float new_obs_weight, int by_count);
void thresh_adapter_setup_2(thresh_adapter *self, double targetspec, double obsper, int NS, int NM, int NL);
void thresh_adapter_setup_3(thresh_adapter *self, double targetspec, double obsper, int NO);
void thresh_adapter_setup_4(thresh_adapter *self, double thresh, double obsper);
void thresh_adapter_start_time(thresh_adapter *self, time_t now);
int thresh_adapter_new_time(thresh_adapter *self, spade_enviro *enviro, double *sugg_thresh);
void thresh_adapter_new_score(thresh_adapter *self, double anom_score);

void thresh_adapter_print_config_details(thresh_adapter *self,FILE *f,char *indent);

/*@}*/
#endif // THRESH_ADAPTER_H

/*********************************************************************
anomscore_surveyer.h, distributed as part of Spade v030125.1
Author: James Hoagland, Silicon Defense (hoagland@SiliconDefense.com)
copyright (c) 2002 by Silicon Defense (http://www.silicondefense.com/)
Released under GNU General Public License, see the COPYING file included
with the distribution or http://www.silicondefense.com/spice/ for details.

anomscore_surveyer.h is the header file for anomscore_surveyer.c

Please send complaints, kudos, and especially improvements and bugfixes to
hoagland@SiliconDefense.com.  As described in GNU General Public License, no
warranty is expressed for this program.
*********************************************************************/

/* Internal version control: Id: anomscore_surveyer.h,v 1.6 2002/12/19 22:37:10 jim Exp $ */

#ifndef ANOMSCORE_SURVEYER_H
#define ANOMSCORE_SURVEYER_H

/*! \file anomscore_surveyer.h
 * \brief 
 *  anomscore_surveyer.h is the header file for anomscore_surveyer.c
 * \ingroup stmgr
 */

/*! \weakgroup stmgr
    @{
*/




#include <time.h>
#include <stdio.h>

/// represents an instance of an anomscore_surveyer
typedef struct {
    /// the number of seconds in the survey interval
    float interval;
    /// the survey file name
    char *filename;

    /// the survey log file handle
    FILE *surveyfile;
    /// the list of anomaly scores for the survey
    ll_double *list;
    /// the length of the list (1-based)
    int list_len;
    /// the suvery period number (starts with 1)
    int period;
    
    /// the start time for this survey interval
    time_t interval_start_time;
    /// the number of packets seen in this survey period so far
    int rec_count;
} anomscore_surveyer;


int init_anomscore_surveyer(anomscore_surveyer *self, char *filename, float interval,spade_msg_fn msg_callback);
int init_anomscore_surveyer_from_str(anomscore_surveyer *self,char *str,spade_msg_fn msg_callback);
anomscore_surveyer *new_anomscore_surveyer(char *filename, float interval,spade_msg_fn msg_callback);
void anomscore_surveyer_flush(anomscore_surveyer *self);
void anomscore_surveyer_shutdown(anomscore_surveyer *self);
void anomscore_surveyer_new_time(anomscore_surveyer *self, spade_enviro *enviro);
void anomscore_surveyer_new_score(anomscore_surveyer *self, double anom_score);

void anomscore_surveyer_print_config_details(anomscore_surveyer *self,FILE *f,char *indent);

/*@}*/
#endif // ANOMSCORE_SURVEYER_H

/*********************************************************************
spade_state.h, distributed as part of Spade v030125.1
Author: James Hoagland, Silicon Defense (hoagland@SiliconDefense.com)
copyright (c) 2002 by Silicon Defense (http://www.silicondefense.com/)
Released under GNU General Public License, see the COPYING file included
with the distribution or http://www.silicondefense.com/spice/ for details.

Please send complaints, kudos, and especially improvements and bugfixes to
hoagland@SiliconDefense.com.  As described in GNU General Public License, no
warranty is expressed for this program.
*********************************************************************/

#ifndef SPADE_STATE_H
#define SPADE_STATE_H

/*! \file spade_state.h
 * \brief 
 *  spade_state.h is the header file for spade_state.c.
 * \ingroup staterec
 */

/*! \addtogroup staterec
    @{
*/

#include <stdio.h>
#include <time.h>

/// a handle for ths user on a currently active state recovery file
typedef struct {
    FILE *f; ///< the file pointer
} statefile_ref;


statefile_ref *spade_state_begin_checkpointing(char *filename, char *appname, u8 app_cur_fvers);
int spade_state_end_checkpointing(statefile_ref *s);
int spade_state_checkpoint_str(statefile_ref *s, char *str);
int spade_state_checkpoint_arr(statefile_ref *s, void *arr, int len, int elsize);
int spade_state_checkpoint_str_arr(statefile_ref *s, char **arr, int len);
int spade_state_checkpoint_u32(statefile_ref *s, u32 val);
int spade_state_checkpoint_u8(statefile_ref *s, u8 val);
int spade_state_checkpoint_time_t(statefile_ref *s, time_t val);
int spade_state_checkpoint_double(statefile_ref *s,double val);
int spade_state_end_section(statefile_ref *s);

statefile_ref *spade_state_begin_recovery(char *filename, int min_app_fvers, char **appname, u8 *file_app_fvers);
int spade_state_end_recovery(statefile_ref *s);
int spade_state_recover_check_end_of_section(statefile_ref *s, int *res);
int spade_state_recover_arr(statefile_ref *s, void *arr, int len, int elsize);
int spade_state_recover_str_arr(statefile_ref *s, char **arr, int len);
int spade_state_recover_u32(statefile_ref *s, u32 *val);
int spade_state_recover_u8(statefile_ref *s, u8 *val);
int spade_state_recover_time_t(statefile_ref *s, time_t *val);
int spade_state_recover_double(statefile_ref *s, double *val);
int spade_state_recover_str(statefile_ref *s, char **str);
int spade_state_recover_str_to_buff(statefile_ref *s, char *buff, int maxlen);

#endif // SPADE_STATE_H

/* Id: spade_state.h,v 1.6 2003/01/14 17:45:31 jim Exp $ */
//
/*********************************************************************
score_info.h, distributed as part of Spade v030125.1
Author: James Hoagland, Silicon Defense (hoagland@SiliconDefense.com)
copyright (c) 2002 by Silicon Defense (http://www.silicondefense.com/)
Released under GNU General Public License, see the COPYING file included
with the distribution or http://www.silicondefense.com/spice/ for details.

Please send complaints, kudos, and especially improvements and bugfixes to
hoagland@SiliconDefense.com.  As described in GNU General Public License, no
warranty is expressed for this program.
*********************************************************************/

/* Internal version control: Id: score_info.h,v 1.6 2002/12/20 06:10:03 jim Exp $ */

#ifndef SCORE_INFO_H
#define SCORE_INFO_H

/*! \file score_info.h
 * \brief 
 *  score_info.h contains the type declaration for the score_info
 *  struct and the interface to the associated functions
 * \ingroup scoreprod
 */

/*! \addtogroup scoreprod
    @{
*/

/// a special double indicating that there is no anomaly score
#define NO_SCORE (double)-1

/// enum containing the possible preferences among types of anomaly scores
typedef enum {PREF_NOSCORE,PREF_RAWSCORE,PREF_RELSCORE} scorepref;

const char *scorepref_str(scorepref pref);

/// stores information about a anomaly scoring result
typedef struct _score_info {
    scorepref main; ///< the preference for which available score to use as the main score
    double relscore; ///< the relative anomaly score, or NO_SCORE
    double rawscore; ///< the raw anomaly score, or NO_SCORE
    int corrscore_used; ///< if the raw anomaly score was computed, was it computed correctly
    
    struct _score_info *next; ///< the next score_info in a list of them
} score_info;

score_info *new_score_info(scorepref main, double relscore, double rawscore, int corrscore_used);
void init_score_info(score_info *i, scorepref main, double relscore, double rawscore, int corrscore_used);
score_info *score_info_clone(score_info *i);
void free_score_info(score_info *i);
void free_score_infos(score_info *start);

double score_info_mainscore(score_info *i);
double score_info_relscore(score_info *i);
double score_info_rawscore(score_info *i);
int score_info_raw_is_corrscore(score_info *i);
scorepref score_info_main_pref(score_info *i);

/*@}*/
#endif // SCORE_INFO_H

/*********************************************************************
spade_event.h, distributed as part of Spade v030125.1
Author: James Hoagland, Silicon Defense (hoagland@SiliconDefense.com)
copyright (c) 2002 by Silicon Defense (http://www.silicondefense.com/)
Released under GNU General Public License, see the COPYING file included
with the distribution or http://www.silicondefense.com/spice/ for details.

Please send complaints, kudos, and especially improvements and bugfixes to
hoagland@SiliconDefense.com.  As described in GNU General Public License, no
warranty is expressed for this program.
*********************************************************************/

/* Internal version control: Id: spade_event.h,v 1.7 2002/12/21 01:25:39 jim Exp $ */

#ifndef SPADE_EVENT_H
#define SPADE_EVENT_H

/*! \file spade_event.h
 * \brief 
 *  spade_event.h contains the type declaration for the spade_event struct 
 *  and the interface to the associated functions
 * \ingroup libspade_misc
 */

/*! \addtogroup libspade_misc
    @{
*/

/*********************************************************************
spade_prob_table_types.h, distributed as part of Spade v030125.1
Author: James Hoagland, Silicon Defense (hoagland@SiliconDefense.com)
copyright (c) 2002 by Silicon Defense (http://www.silicondefense.com/)
Released under GNU General Public License, see the COPYING file included
with the distribution or http://www.silicondefense.com/spice/ for details.

Please send complaints, kudos, and especially improvements and bugfixes to
hoagland@SiliconDefense.com.  As described in GNU General Public License, no
warranty is expressed for this program.
*********************************************************************/

#ifndef SPADE_PROB_TABLE_TYPES_H
#define SPADE_PROB_TABLE_TYPES_H

/*! \file spade_prob_table_types.h
 * \brief 
 *  spade_prob_table_types.h is the header file for spade_prob_table_types.c.
 * \ingroup staterec
 */

/*! \addtogroup staterec
    @{
*/



/// index type into tree memory block data structures
typedef u32 mindex;

/// dmindex is a mindex used with top bit indicating if one of two datatypes is present
typedef u32 dmindex;

/// the type of the values of the features
/** \note right now, we assume all features can be contained in a u32 and can be sorted as unsigned ints; we may need to extend this someday */
typedef u32 valtype;


/// a tree root
typedef struct _treeroot {
    mindex next;      ///< the next tree root in a list
    dmindex root;     ///< root node of the tree, if top bit is 1, it is a leafnode, otherwise it is a interior node
    features type;    ///< the feature that is being represented in this tree
    double entropy;   ///< the last calculated entropy in this tree; < 0 if it has not been calculated
    u16 entropy_wait; ///< the number of additions to the tree to wait till recalculating the entropy; only valid if entropy >= 0
} treeroot;

/// an interior node in the tree
typedef struct _intnode {
    double sum;     ///< the sum of the counts underneath this node in the tree
    valtype sortpt; ///< the highest value on the left side of this node
    dmindex left;   ///< the left node; if top bit is 1, it is a leafnode
    dmindex right;  ///< the right node; if top bit is 1, it is a leafnode
    u16 wait;       ///< the number of additions to the subtree to wait before checking for reblancing
} intnode;

/// a leaf node of the tree
typedef struct _leafnode {
    double count;    ///< the count on this node
    valtype value;   ///< the value this node represents
    mindex nexttree; ///< the first in a linked list of trees anchored from this leaf node
} leafnode;

#define isleaf(node) (node & DMINDEXMASK)
#define asleaf(leaf) (leaf | DMINDEXMASK)
#define encleaf2mindex(node) (node ^ DMINDEXMASK)
/* arg is a dmindex; if it denotes a leaf, return the count on that leaf
   otherwise return the sum on the interior node */ 
#define count_or_sum(node) (isleaf(node) ? leafnode(encleaf2mindex(node)).count : intnode(node).sum)
#define eleafval(leaf) leafnode(encleaf2mindex(leaf)).value
#define largestval(node) (isleaf(node) ? eleafval(node) : largest_val(node))
#define treetype(t) tree(t).type
#define treeroot(t) tree(t).root
#define treenext(t) tree(t).next
#define treeH(t) tree(t).entropy
#define treeH_wait(t) tree(t).entropy_wait
#define intleft(node) intnode(node).left
#define intright(node) intnode(node).right
#define intsum(node) intnode(node).sum
#define intsortpt(node) intnode(node).sortpt
#define intwait(node) intnode(node).wait
#define leafcount(leaf) leafnode(leaf).count
#define leafvalue(leaf) leafnode(leaf).value
#define leafnexttree(leaf) leafnode(leaf).nexttree


/* defaults unless recovering from a checkpoint */
#define DEFAULT_ROOT_BLOCK_BITS 10
#define DEFAULT_INT_BLOCK_BITS 9
#define DEFAULT_LEAF_BLOCK_BITS 10

/* these number of blocks are used
   unless file recovering from already uses more blocks */
#define DEFAULT_MAX_ROOT_BLOCKS 4500
#define DEFAULT_MAX_INT_BLOCKS 12000
#define DEFAULT_MAX_LEAF_BLOCKS 9000

#define bits2blocksize(b) (1 << b)

#define ROOT_BLOCK_SIZE bits2blocksize(ROOT_BLOCK_BITS)
#define ROOT_BLOCK_MASK ((1 << ROOT_BLOCK_BITS) -1)
#define tree(i) ROOT_M[i>>ROOT_BLOCK_BITS][i&ROOT_BLOCK_MASK]
#define root_index(p,i) ((p<<ROOT_BLOCK_BITS)+i)

#define INT_BLOCK_SIZE bits2blocksize(INT_BLOCK_BITS)
#define INT_BLOCK_MASK ((1 << INT_BLOCK_BITS) -1)
#define intnode(i) INT_M[i>>INT_BLOCK_BITS][i&INT_BLOCK_MASK]
#define intnode_index(p,i) ((p<<INT_BLOCK_BITS)+i)

#define LEAF_BLOCK_SIZE bits2blocksize(LEAF_BLOCK_BITS)
#define LEAF_BLOCK_MASK ((1 << LEAF_BLOCK_BITS) -1)
#define leafnode(i) LEAF_M[i>>LEAF_BLOCK_BITS][i&LEAF_BLOCK_MASK]
#define leafnode_index(p,i) ((p<<LEAF_BLOCK_BITS)+i)

#define rfreenext(n) (n).next
#define ifreenext(n) (n).left
#define lfreenext(n) (n).nexttree

/* something of valtype that cannot be a sortpt */
#define NOT_A_SORTPT ((u32)MAX_U32)

#define TNULL (mindex)-1
#define DMINDEXMASK ((dmindex)(1 << (sizeof(dmindex)*8-1)))

extern treeroot **ROOT_M;
extern intnode **INT_M;
extern leafnode **LEAF_M;
extern mindex root_freelist;
extern mindex int_freelist;
extern mindex leaf_freelist;

void init_mem();
void allocate_mem_blocks();
int reallocate_ptr_array(void ***arrptr,int oldsize,int newsize);

mindex new_treeinfo(features type);
void free_treeinfo(mindex f);
mindex new_int();
void free_int(mindex f);
mindex new_leaf(valtype val);
void free_leaf(mindex f);

extern unsigned char ROOT_BLOCK_BITS;
extern unsigned char INT_BLOCK_BITS;
extern unsigned char LEAF_BLOCK_BITS;
extern unsigned int MAX_ROOT_BLOCKS;
extern unsigned int MAX_INT_BLOCKS;
extern unsigned int MAX_LEAF_BLOCKS;

#endif // SPADE_PROB_TABLE_TYPES_H

/* Id: spade_prob_table_types.h,v 1.8 2003/01/08 19:59:54 jim Exp $ */


/// function type to make a copy of a spade_event's "native" field
typedef void *(*event_native_copier_t)(void *native);
/// function type to free a copy of a spade_event's "native" field
typedef void (*event_native_freer_t)(void *native);

/// the representation of an event that is being given to libspade
typedef struct {
    /// the features, indexed by feature number (libspade-user assigned)
    valtype fldval[MAX_NUM_FEATURES];
    /// the time the event occurred
    double time;
    /// the origin of the event, in user defined terms
    u32 origin;

    /// a field the user may use to maintain a reference to its own representation of the event
    void *native;
    /// used internally, at times this field may be set to point to a routine to free the native copy
    event_native_freer_t native_freer;
} spade_event;

spade_event *new_spade_event(void);
spade_event *spade_event_clone(spade_event *e, event_native_copier_t native_copier, event_native_freer_t native_freer);
void free_spade_event(spade_event *e);

/*@}*/

#endif // SPADE_EVENT_H




#define ADVISING_OFF     0 ///< threshold advising status indicating the threshold advising was never on
#define ADVISING_RUNNING 1 ///< threshold advising status indicating the threshold advising is on and running
#define ADVISING_DONE    2 ///< threshold advising status indicating the threshold advising was on but has now completed

/// function type for a function to call when a score exceeds the reporting threshold
typedef void (*spade_thresh_exceeded_fn_t)(void *context,void *targetref,spade_event *event,score_info *score);
/// function type for a function to call when a reporting threshold changes
typedef void (*spade_thresh_changed_fn_t)(void *context,void *targetref);

// storage for an instance of a score manager
typedef struct {
    void *mgrref; ///< some sort of identifier of this score manager
    spade_enviro *enviro; ///< environment shared with target user
    
    int adapt_active; ///< 0 is there is no adapting; otherwise the adapt method #
    int advise_status; ///< ADVISING_OFF, ADVISING_RUNNING, or ADVISING_DONE
    int survey_active; ///< true if survey mode is active
    
    thresh_adapter adapter; ///< our threshold adapter module, if threshold adapting is in use
    thresh_adviser adviser; ///< our threshold advising module, if threshold advising is in use
    anomscore_surveyer surveyer; ///< our anomaly score surveyer module, if this is on
        
    /// function to call when we see a score that exceeds the threshold, or NULL if none
    spade_thresh_exceeded_fn_t threshexceeded_callback;
    /// function to call when the reporting threshold changes, or NULL if none
    spade_thresh_changed_fn_t threshchanged_callback;
    void *callback_context; ///< user-provided pointer that is returned as first arg in callback

    spade_msg_fn msg_callback; ///< function to call when there is a message for the user
} score_mgr;

score_mgr *new_score_mgr(void *mgrref, spade_enviro *enviro, void *callback_context, spade_thresh_exceeded_fn_t threshexceeded_callback, spade_thresh_changed_fn_t threshchanged_callback,spade_msg_fn msg_callback);
void init_score_mgr(score_mgr *self, void *mgrref, spade_enviro *enviro, void *callback_context, spade_thresh_exceeded_fn_t threshexceeded_callback, spade_thresh_changed_fn_t threshchanged_callback,spade_msg_fn msg_callback);

void score_mgr_setup_adapt_from_str(score_mgr *self, int adaptmode, char *str);
void score_mgr_setup_adapt1(score_mgr *self, int target, time_t period, float new_obs_weight, int by_count);
void score_mgr_setup_adapt2(score_mgr *self, double targetspec, double obsper, int NS, int NM, int NL);
void score_mgr_setup_adapt3(score_mgr *self, double targetspec, double obsper, int NO);
void score_mgr_setup_adapt4(score_mgr *self, double thresh, double obsper);
void score_mgr_setup_advise(score_mgr *self, int obs_size, int obs_secs);
void score_mgr_setup_advise_from_str(score_mgr *self, char *str);
void score_mgr_setup_survey(score_mgr *self, char *filename, float interval);
void score_mgr_setup_survey_from_str(score_mgr *self, char *str);

int score_mgr_new_time(score_mgr *self, time_t time);
void score_mgr_new_event(score_mgr *self, score_info *score, spade_event *event);

void score_mgr_dump(score_mgr *self);
void score_mgr_cleanup(score_mgr *self);
void score_mgr_file_print_log(score_mgr *self, FILE *file);

void score_mgr_print_config_details(score_mgr *self, FILE *file, char *indent);

/*@}*/
#endif // SCORE_MGR_H


/*********************************************************************
spade_report.h, distributed as part of Spade v030125.1
Author: James Hoagland, Silicon Defense (hoagland@SiliconDefense.com)
copyright (c) 2002 by Silicon Defense (http://www.silicondefense.com/)
Released under GNU General Public License, see the COPYING file included
with the distribution or http://www.silicondefense.com/spice/ for details.

spade_report.h contains the type declaration for the spade_report struct 
  and the interface to the associated functions

Please send complaints, kudos, and especially improvements and bugfixes to
hoagland@SiliconDefense.com.  As described in GNU General Public License, no
warranty is expressed for this program.
*********************************************************************/

/* Internal version control: Id: spade_report.h,v 1.7 2003/01/08 19:59:54 jim Exp $ */

#ifndef SPADE_REPORT_H
#define SPADE_REPORT_H

/*! \file spade_report.h
 * \ingroup netspade_layer
 * \brief 
 *  spade_report.h contains the type declaration for the spade_report struct 
 *  and the interface to the associated functions
 */

/*! \addtogroup netspade_layer
    @{
*/




#include <stdio.h>

/// a mask for the bits of a port_status_t that represent belief strength
#define PORT_STRENGTH_MASK 0x003
/// a mask for the bits of a port_status_t that represent the port status
#define PORT_BASE_MASK (0xFFF & ~PORT_STRENGTH_MASK)

/// mask off the port status part of a port_status_t
#define PORT_BASE(status) ((status) & PORT_BASE_MASK)
/// mask off the belief strength part of a port_status_t
#define PORT_STRENGTH(status) ((status) & PORT_STRENGTH_MASK)

/// representation of a definate strength belief in port status
#define PORT_STRENGTH_DEFINATE 0x002
/// representation of a "likely" strength belief in port status
#define PORT_STRENGTH_LIKELY 0x001
/// representation of a "probably" strength belief in port status
#define PORT_STRENGTH_PROBABLY 0x000

/// port status part of a port_status_t when it is open
#define PORT_OPEN_BASE (1<<2)
/// port status part of a port_status_t when it is closed
#define PORT_CLOSED_BASE (2<<2)

/// an enum of value that represents our preception of a port's status
/** the integer value is based on a combination of port status and strength of belief */
typedef enum {
    /// we don't know the port's status
    PORT_UNKNOWN=0,
    /// we think the port is probably open
    PORT_PROBOPEN       =PORT_OPEN_BASE  |PORT_STRENGTH_PROBABLY,
    /// we think the port is likely open
    PORT_LIKELYOPEN     =PORT_OPEN_BASE  |PORT_STRENGTH_LIKELY,
    /// we think the port is definately open
    PORT_OPEN           =PORT_OPEN_BASE  |PORT_STRENGTH_DEFINATE,
    /// we think the port is probably closed
    PORT_PROBCLOSED     =PORT_CLOSED_BASE|PORT_STRENGTH_PROBABLY,
    /// we think the port is likely closed
    PORT_LIKELYCLOSED   =PORT_CLOSED_BASE|PORT_STRENGTH_LIKELY,
    /// we think the port is definately closed
    PORT_CLOSED         =PORT_CLOSED_BASE|PORT_STRENGTH_DEFINATE
} port_status_t;

#define PORT_STATUS_AS_STR(status) (status == PORT_UNKNOWN ? "PORT_UNKNOWN" \
    : (status == PORT_PROBOPEN) ? "PORT_PROBOPEN" \
    : (status == PORT_LIKELYOPEN) ? "PORT_LIKELYOPEN" \
    : (status == PORT_OPEN) ? "PORT_OPEN" \
    : (status == PORT_PROBCLOSED) ? "PORT_PROBCLOSED" \
    : (status == PORT_LIKELYCLOSED) ? "PORT_LIKELYCLOSED" \
    : (status == PORT_PROBOPEN) ? "PORT_PROBOPEN" \
    : (status == PORT_CLOSED) ? "PORT_CLOSED" \
    : "UNDEFINED" \
    )

/// data type for representing a set of port statuses
/** the value is based on a bit mask, where the bit position is determined by the value of the port_status_t */
typedef u16 port_status_set_t;
/// port_status_set_t value when the set is empty
#define PS_EMPTY_SET (port_status_set_t)0
/// port_status_set_t value when the set consists of just the given port_status_t value
#define PS_STATUS_MASK(status) (1 << (status))
/// port_status_set_t value when the set consists of the given port_status_t value and stronger beliefs
#define PS_STATUS_MASK_WITH_STRONGER(status) ((status) == PORT_UNKNOWN ? 0xFFF \
    : PS_STATUS_MASK(status) \
        | (PORT_STRENGTH(status) < PORT_STRENGTH_LIKELY \
            ? (PS_STATUS_MASK(PORT_BASE(status)|PORT_STRENGTH_LIKELY)) \
            : 0 ) \
        | (PORT_STRENGTH(status) < PORT_STRENGTH_DEFINATE \
            ? (PS_STATUS_MASK(PORT_BASE(status)|PORT_STRENGTH_DEFINATE)) \
            : 0 ) \
    )
/// initialize a port_status_set_t variable to contatin just the given port_status_t value
#define PS_INIT_SET(set,status) (set= PS_STATUS_MASK(status))
/// initialize a port_status_set_t variable to contatin of the given port_status_t value and stronger beliefs
#define PS_INIT_SET_WITH_STRONGER(set,status) (set= PS_STATUS_MASK_WITH_STRONGER(status))
/// add the given port_status_t to a port_status_set_t stored in the given variable
#define PS_ADD_TO_SET(set,status) (set|= PS_STATUS_MASK(status))
/// add the given port_status_t and stronger beliefs to a port_status_set_t stored in the given variable
#define PS_ADD_TO_SET_WITH_STRONGER(set,status) (set|= PS_STATUS_MASK_WITH_STRONGER(status))
/// test if the given port_status_t in the given port_status_set_t
#define PS_IN_SET(set,status) (set & PS_STATUS_MASK(status))

void port_status_set_file_print(port_status_set_t set,FILE *f);

/// the representation of a Spade report internally and to the libnetspade user
typedef struct _spade_report {
    /// the detector type triggering this spade report (from spade_detection_types.h)
    int detect_type;
    /// the detector id of the spade detector that is sending this report
    char *detectorid;
    /// pointer to the packet (originally provided by the libspade user) this report is about
    spade_event *pkt;
    /// pointer to the calculated packet anomaly score(s)
    score_info *score;
    /// current port status
    port_status_t port_status;
    /// string representing the detection type employed
    const char *detect_type_str;
    /// string representing the detectors scope
    char scope_str[200];
    /// pointer to stream statistics
    spade_pkt_stats *stream_stats;
    /// the next spade report in a list of them
    struct _spade_report *next;
} spade_report;

spade_report *new_spade_report(spade_event *pkt,score_info *score, int detect_type, char *detectorid,const char *detect_type_str,char *scope_str,spade_pkt_stats *stream_stats,port_status_t port_status);
void free_spade_report(spade_report *rpt);
void free_spade_reports(spade_report *rpt);

#define spade_report_mainscore(rpt) (rpt != NULL ? score_info_mainscore(rpt->score) : NO_SCORE)
#define spade_report_relscore(rpt) (rpt != NULL ? score_info_relscore(rpt->score) : NO_SCORE)
#define spade_report_rawscore(rpt) (rpt != NULL ? score_info_rawscore(rpt->score) : NO_SCORE)
#define spade_report_raw_is_corrscore(rpt) (score_info_raw_is_corrscore(rpt->score))
#define spade_report_main_pref(rpt) (score_info_main_pref(rpt->score))

/*@}*/

#endif // SPADE_REPORT_H

/*********************************************************************
score_calculator.h, distributed as part of Spade v030125.1
Author: James Hoagland, Silicon Defense (hoagland@SiliconDefense.com)
copyright (c) 2002 by Silicon Defense (http://www.silicondefense.com/)
Released under GNU General Public License, see the COPYING file included
with the distribution or http://www.silicondefense.com/spice/ for details.

Please send complaints, kudos, and especially improvements and bugfixes to
hoagland@SiliconDefense.com.  As described in GNU General Public License, no
warranty is expressed for this program.
*********************************************************************/

/* Internal version control: Id: score_calculator.h,v 1.8 2002/12/20 06:10:03 jim Exp $ */

#ifndef SCORE_CALCULATOR_H
#define SCORE_CALCULATOR_H

/*! \file score_calculator.h
 * \brief 
 *  score_calculator.h is the header file for score_calculator.c
 * \ingroup scoreprod
 */

/*! \addtogroup scoreprod
    @{
*/

#include <stdio.h>

/*********************************************************************
event_recorder.h, distributed as part of Spade v030125.1
Author: James Hoagland, Silicon Defense (hoagland@SiliconDefense.com)
copyright (c) 2002 by Silicon Defense (http://www.silicondefense.com/)
Released under GNU General Public License, see the COPYING file included
with the distribution or http://www.silicondefense.com/spice/ for details.

Please send complaints, kudos, and especially improvements and bugfixes to
hoagland@SiliconDefense.com.  As described in GNU General Public License, no
warranty is expressed for this program.
*********************************************************************/

/* Internal version control: Id: event_recorder.h,v 1.8 2003/01/23 17:41:21 jim Exp $ */

#ifndef EVENT_RECORDER_H
#define EVENT_RECORDER_H

/*! \file event_recorder.h
 * \brief 
 *  event_recorder.h is the header file for event_recorder.c
 * \ingroup staterec
 */

/*! \addtogroup staterec
    @{
*/


/*********************************************************************
spade_prob_table.h, distributed as part of Spade v030125.1
Author: James Hoagland, Silicon Defense (hoagland@SiliconDefense.com)
copyright (c) 2002 by Silicon Defense (http://www.silicondefense.com/)
Released under GNU General Public License, see the COPYING file included
with the distribution or http://www.silicondefense.com/spice/ for details.

Please send complaints, kudos, and especially improvements and bugfixes to
hoagland@SiliconDefense.com.  As described in GNU General Public License, no
warranty is expressed for this program.
*********************************************************************/

#ifndef SPADE_PROB_TABLE_H
#define SPADE_PROB_TABLE_H

/*! \file spade_prob_table.h
 * \brief 
 *  spade_prob_table.h is the header file for spade_prob_table.c.
 * \ingroup staterec
 */

/*! \addtogroup staterec
    @{
*/





#include <stdio.h>

/// a spade probability table, represented by some number of nested trees
typedef struct {
    mindex root[MAX_NUM_FEATURES]; ///< top level tree roots in an array indexed by the feature type of the top level tree
    const char **featurenames;     ///< user provided pointer to array of the string names of the features, used for output
} spade_prob_table;

/// an element in a data structure representing a set of doubles indexed by a list of features
typedef struct _featcomb {
    struct _featcomb *next[MAX_NUM_FEATURES]; ///< pointers to the next level of structure, indexed by feature
    double val[MAX_NUM_FEATURES];  ///< the values stored; indexed by the final feature in the list
} *featcomb;

#define STATS_NONE          0x00  ///< no statistics
#define STATS_ENTROPY       0x01  ///< entropy statistics
#define STATS_UNCONDPROB    0x02  ///< unconditional probabilities
#define STATS_CONDPROB      0x04  ///< conditional probabilities

#define PROBRESULT_NO_RECORD (double)-1.0 ///< a special probability value denoting the probability denominator was 0


void init_spade_prob_table(spade_prob_table *self,const char **featurenames,int recovering);
spade_prob_table *new_spade_prob_table(const char **featurenames);

int spade_prob_table_is_empty(spade_prob_table *self);

void increment_simple_count(spade_prob_table *self, features type1, valtype val1);
void increment_2joint_count(spade_prob_table *self, features type1, valtype val1, features type2, valtype val2, int skip);
void increment_3joint_count(spade_prob_table *self, features type1, valtype val1, features type2, valtype val2, features type3, valtype val3, int skip);
void increment_4joint_count(spade_prob_table *self, features type1, valtype val1, features type2, valtype val2, features type3, valtype val3, features type4, valtype val4, int skip);
void increment_Njoint_count(spade_prob_table *self,int size,features type[],valtype val[],int skip);

double prob_simple(spade_prob_table *self, features type1, valtype val1);
double prob_2joint(spade_prob_table *self, features type1, valtype val1, features type2, valtype val2);
double prob_Njoint(spade_prob_table *self, int size, features type[], valtype val[]);
double prob_Njoint_Ncond(spade_prob_table *self, int size, features type[], valtype val[], int condoffset);
double prob_Njoint_Ncond_plus_one(spade_prob_table *self, int size, features type[], valtype val[], int condoffset);
double prob_cond1(spade_prob_table *self, features type, valtype val, features ctype, valtype cval);
double prob_cond2(spade_prob_table *self, features type, valtype val, features ctype1, valtype cval1, features ctype2, valtype cval2);
double prob_cond3(spade_prob_table *self, features type, valtype val, features ctype1, valtype cval1, features ctype2, valtype cval2, features ctype3, valtype cval3);
double one_prob_simple(spade_prob_table *self,features type1);

double jointN_count(spade_prob_table *self,int size,features type[], valtype val[]);

double spade_prob_table_entropy(spade_prob_table *self, int size, features type[], valtype val[]);

void scale_and_prune_table(spade_prob_table *self, double factor, double threshold);

float feature_trees_stats(spade_prob_table *self, features f, float *amind, float *amaxd, float *aaved, float *awaved);

void spade_prob_table_write_stats(spade_prob_table *self,FILE *file,u8 stats_to_print);
void write_feat_val_list(spade_prob_table *self,FILE *f, int depth, features feats[], valtype vals[]);
void write_all_uncond_probs(spade_prob_table *self, FILE *f);
void write_all_cond_probs(spade_prob_table *self, FILE *f);
featcomb calc_all_entropies(spade_prob_table *self);
void write_all_entropies(spade_prob_table *self,FILE *f, featcomb c);

void print_spade_prob_table(spade_prob_table *self);
int sanity_check_spade_prob_table(spade_prob_table *self);


int spade_prob_table_checkpoint(statefile_ref *s,spade_prob_table *self);
int spade_prob_table_recover(statefile_ref *s,spade_prob_table *self);


#endif // SPADE_PROB_TABLE_H

/* Id: spade_prob_table.h,v 1.7 2003/01/08 19:59:54 jim Exp $ */





#include <stdio.h>

/// a set of event conditions
/** An event condition is a boolean condition on a spade_event; it is true
    if the condition is satisfied and false otherwise.  All regular event
    conditions are defined by the libspade user.  event_condition_set
    represents a set of conditions which are simulatneously true for an
    event.  This is presently represented as a u32 bit-mask, which is
    operationally very efficient but which has its limits including only
    being able to represent 32 conditions and permitting only boolean
    conditions */
typedef u32 event_condition_set;
/// macro to get at event condition #n
#define EVENT_CONDITION_NUM(n) (1 << (n-1))
/// the special true event condition
#define EVENT_CONDITION_TRUE 0
/// the special false event condition
#define EVENT_CONDITION_FALSE EVENT_CONDITION_NUM(32)

/// are all conditions in the event_condition_set ref met in the testcase event_condition_set
#define ALL_CONDS_MET(testcase,ref) (((testcase) & (ref)) == (ref))
/// is at least one of the conditions in ref met in the testcase
#define SOME_CONDS_MET(testcase,ref) ((testcase) & (ref))
/// add newconds to event_condition_set var
#define ADD_TO_CONDS(var,newconds) ((var) |= (newconds))
/// remove remconds from the event_condition_set var
#define REMOVE_FROM_CONDS(var,remconds) ((var) &= ~remconds)
/// compose a condition set (returned) from N event conditions
#define CONDS_PLUS_CONDS(cond1,cond2) (cond1 | cond2)
#define CONDS_PLUS_2CONDS(cond1,cond2,cond3) (cond1 | cond2 | cond3)
#define CONDS_PLUS_3CONDS(cond1,cond2,cond3,cond4) (cond1 | cond2 | cond3 | cond4)
#define CONDS_PLUS_4CONDS(cond1,cond2,cond3,cond4,cond5) (cond1 | cond2 | cond3 | cond4 | cond5)
#define CONDS_PLUS_5CONDS(cond1,cond2,cond3,cond4,cond5,cond6) (cond1 | cond2 | cond3 | cond4 | cond5 | cond6)
#define CONDS_PLUS_6CONDS(cond1,cond2,cond3,cond4,cond5,cond6,cond7) (cond1 | cond2 | cond3 | cond4 | cond5 | cond6 | cond7)
/// return the event_condition_set formed by removing cond2 from cond1
#define CONDS_MINUS_CONDS(cond1,cond2) (cond1 & ~cond2)
/// test if the condition set indicates false
#define CONDS_NOT_FALSE(conds) (((conds) & EVENT_CONDITION_FALSE) == 0)
/// return the event_condition_set formed by restricting cond1 to only those conditions in cond2
#define ONLY_CONDS(origconds,onlyconds) ((origconds) & onlyconds)


/// structure containing the elements of a table manager
typedef struct _table_mgr {
    spade_prob_table table; ///< the probability table we use
    struct _table_mgr *next; ///< the next table manager in a linked list of them
    time_t last_scale; ///< the last time this table was scaled/pruned
    u32 store_count; ///< the number of events we have stored
    
    feature_list feats; ///< the features we store, in order
    const char **featurenames; ///< direct-index lookup array to map features to their names
    
    event_condition_set conds; ///< the event conditions under which this table is used for storage
    time_t start_time; ///< the time we started recording events
    int scale_freq; ///< how often we scale/prune, in secs
    double scale_factor; ///< when we scale, how much do we do so?; this is the multiplier
    double prune_threshold;  ///< if an observation gets below this size, it gets discarded
    int use_count; ///< how many event files are using this table manager
} table_mgr;

/// structure containing the elements on an event file
typedef struct _evfile {
    /// the table manager that does the storge for this event file
    table_mgr *mgr;
    /// this how many features deep we care about; table manager may store more for other reasons
    int feat_depth;
    /// the features we need stored, in order
    feature_list calc_feats; /*!< \note maybe eventually we should maintain a user-addressable set of these */

    /// the next event file in a linked list of them
    struct _evfile *next;
} evfile;

/// the way a event_recorder user refers to a particular event file
typedef evfile *evfile_ref;

/// the representation for an instance of an event_recorder
typedef struct {
    /// linked list of the table managers that are defined
    table_mgr *tables;
    /// linked list of the event files that are defined
    evfile *files;
    /// the current time
    time_t curtime;
} event_recorder;

/// function type that can be called to print the string version of a set of event conditions to a FILE *
typedef void (*condition_printer_t)(FILE *file,event_condition_set conds);


event_recorder *new_event_recorder(void);
void init_event_recorder(event_recorder *self);

int event_recorder_recover(event_recorder **self, statefile_ref *ref);
int event_recorder_merge_recover(event_recorder *self, statefile_ref *ref);
int event_recorder_checkpoint(event_recorder *self, statefile_ref *ref);

evfile_ref event_recorder_new_event_file(event_recorder *self, feature_list *feats, const char **featurenames, event_condition_set conds, int scale_freq, double scale_factor, double prune_threshold, int fresh_only, feature_list *calc_feats);
evfile_ref *event_recorder_new_event_files(event_recorder *self, int howmany, feature_list feats[], const char **featurenames, event_condition_set conds, int scale_freq, double scale_factor, double prune_threshold, int fresh_only);

void event_recorder_new_time(event_recorder *self, time_t time);
event_condition_set event_recorder_needed_conds(event_recorder *self);
int event_recorder_new_event(event_recorder *self, spade_event *event, event_condition_set matching_conds);
void event_recorder_prune_unused(event_recorder *self);

double event_recorder_get_prob(event_recorder *self, evfile_ref eventfile, spade_event *event,int one_more);
double event_recorder_get_condprob(event_recorder *self, evfile_ref eventfile, spade_event *event, int condcutoff,int one_more);
double event_recorder_get_count(event_recorder *self, evfile_ref eventfile, spade_event *event, int featdepth);
double event_recorder_get_entropy(event_recorder *self,evfile_ref eventfile,spade_event *event,int entropy_prefix_len);

int event_recorder_get_store_count(event_recorder *self, evfile_ref eventfile);
double event_recorder_get_obs_count(event_recorder *self, evfile_ref eventfile);

void event_recorder_write_stats(event_recorder *self, FILE *file, u8 stats_to_print,condition_printer_t condprinter);

void evfile_print_config_details(evfile_ref eventfile,FILE *f,char *indent);
#endif // EVENT_RECORDER_H



/// a structure containing the various components describing how a probability table is going to be used
typedef struct {
    int prodcount; ///< the number of conditional probabilities to multiply
    feature_list *feats; ///< if prodcount > 1, the array of feature lists to use; space is dynamically allocated 
    feature_list calc_feats; ///< if prodcount=1, the list of features for the table
    const char **featurenames; ///< direct-index lookup array to map features to their names
    event_condition_set conds; ///< the event conditions under which the table will be used for storage
    int scale_freq; ///< how often the table will be scaled/pruned, in secs
    double scale_factor; ///< when we scale, how much do we do so by
    double prune_threshold; ///< if an observation gets below this size, it will be discarded
} table_use_specs;

/// an instance of a score calculator
typedef struct {
    int prodcount; ///< the number of conditional probabilities to multiply
    evfile_ref *evfiles; ///< if prodcount > 1, array of evfile's in the event recorder storing the tables needed for each probability
    evfile_ref evfile; ///< if we only have one prob, we store store the evfile here
    int calc_rawscore; ///< should the raw anonamaly score be calculated
    int calc_relscore; ///< should the relative anonamaly score be calculated
    scorepref mainpref; ///< the preferred score type to store
    int use_corrscore; ///< correctly compute the raw anomaly score?
    int cond_prefix_len; ///< how far into feats should we use as the denominator for calculating the probability
    int min_obs_prefix_len; ///< if > 0, we require a min observation, this is how far into feats the spec is for
    double min_obs_count; ///< the minimum observation count
    double max_entropy; ///< if >= 0, we are using a selection critea based on maximum entropy under the values of a certain feature
    int entropy_prefix_len; ///< the depth of the run-up to the value field when using max entropy selection criterea
    table_use_specs *evfiles_data; ///< parameters to evfiles while being set up
    event_recorder *recorder; ///< a pointer to the event recorder where the events are stored 
} score_calculator;

score_calculator *new_score_calculator(int prodcount, feature_list prod_cond[], const char **featurenames, event_condition_set conds, int scale_freq, double scale_factor, double prune_threshold, event_recorder *recorder, feature_list *calc_feats);
void init_score_calculator(score_calculator *self, int prodcount, feature_list prod_cond[], const char **featurenames, event_condition_set conds, int scale_freq, double scale_factor, double prune_threshold, event_recorder *recorder, feature_list *calc_feats);
score_calculator *new_score_calculator_clear(event_recorder *recorder);
void init_score_calculator_clear(score_calculator *self, event_recorder *recorder);

void score_calculator_set_features(score_calculator *self, int prodcount, feature_list prod_cond[], feature_list *calc_feats, const char **featurenames);
void score_calculator_set_storage_conditions(score_calculator *self, event_condition_set conds);
void score_calculator_set_scaling(score_calculator *self, int scale_freq, double scale_factor, double prune_threshold);
void score_calculator_init_complete(score_calculator *self);

void score_calculator_set_condcutoff(score_calculator *self, int cond_prefix_len);
void score_calculator_set_relscore(score_calculator *self, int calc_relscore, int rel_is_main);
void score_calculator_set_rawscore(score_calculator *self, int calc_rawscore, int raw_is_main);
void score_calculator_set_corrscore(score_calculator *self, int use_corrscore);
void score_calculator_set_min_obs(score_calculator *self, int featlist_prefix_len, int min_obs_count);
void score_calculator_set_low_entropy_domain(score_calculator *self, int val_prefix_len, double max_entropy);
void score_calculator_cleanup(score_calculator *self);

int score_calculator_using_corrscore(score_calculator *self);
int score_calculator_using_relscore(score_calculator *self);

score_info *score_calculator_calc_event_score(score_calculator *self, spade_event *event, score_info *storage,int *enoughobs);

int score_calculator_get_store_count(score_calculator *self);
double score_calculator_get_obs_count(score_calculator *self);

void score_calculator_print_config_details(score_calculator *self,FILE *f,char *indent);

/*@}*/
#endif // SCORE_CALCULATOR_H

/*********************************************************************
packet_resp_canceller.h, distributed as part of Spade v030125.1
Author: James Hoagland, Silicon Defense (hoagland@SiliconDefense.com)
copyright (c) 2002 by Silicon Defense (http://www.silicondefense.com/)
Released under GNU General Public License, see the COPYING file included
with the distribution or http://www.silicondefense.com/spice/ for details.

Please send complaints, kudos, and especially improvements and bugfixes to
hoagland@SiliconDefense.com.  As described in GNU General Public License, no
warranty is expressed for this program.
*********************************************************************/

#ifndef PACKET_RESP_CANCELLER_H
#define PACKET_RESP_CANCELLER_H

/*! \file packet_resp_canceller.h
 * \ingroup netspade_layer
 * \brief 
 *  packet_resp_canceller.h is the header file for packet_resp_canceller.c.
 */

/*! \addtogroup netspade_layer
    @{
*/


#include <time.h>

/// function type for a callback for a packet response canceller to report the status of a report
typedef void (*prc_report_status_fn)(void *context,spade_report *rpt,port_status_t status);

/// a two-way link used to store reports in the packet response canceller
typedef struct _prc_link {
    spade_report *rpt; /*!< the report being stored */
    struct _prc_link *ltl_next; /*!< next link in a lookup table list */
    struct _prc_link *ttl_next; /*!< next link in a time table list */
} prc_link;

/// a list of prc_link's
typedef struct {
    prc_link *head;  ///< the head
    prc_link *tail;  ///< the tail
} prc_list;

/// the packet response canceller time-based table
typedef struct {
    prc_list *arr; ///< an array of buckets, each holding the reports for a given second
    int num_buckets; ///< how many buckets are there
    time_t last_timeout; ///< when was the last time a timeout was checked for
} prc_time_table;

#define LOOKUP_TABLE1_BITS 12 ///< how many bits are in the key to the first level lookup table
#define LOOKUP_TABLE2_BITS 8 ///< how many bits are in the key to the second level lookup tables
#define LOOKUP_TABLE1_SIZE (1 << LOOKUP_TABLE1_BITS) ///< the number of elements in the first level lookup hash table
#define LOOKUP_TABLE2_SIZE (1 << LOOKUP_TABLE2_BITS) ///< the number of elements in the second level lookup hash tables
#define LOOKUP_TABLE1_MASK (LOOKUP_TABLE1_SIZE -1) ///< mask used in the hash function for the first level lookup table
#define LOOKUP_TABLE2_MASK (LOOKUP_TABLE2_SIZE -1) ///< mask used in the hash function for the second level lookup tables

/// a second level lookup table
/** this table maps a report to a unsorted linked list of stored reports */
typedef struct {
    prc_link *arr[LOOKUP_TABLE2_SIZE]; ///< a second level hash table, each bucket hold a set reports
    int num_used; ///< the number of slots in the hash table that are presently used
} prc_lookup_table2;

/// the first level lookup table
/** this table maps a report to a second level hash table */
typedef struct {
    prc_lookup_table2 *arr[LOOKUP_TABLE1_SIZE]; ///< a second level hash table, each bucket holds a second level lookup table
} prc_lookup_table;

/// an instance of a packet response canceller, which implements a packet response buffer
typedef struct {
    prc_lookup_table lt; ///< the lookup table, used for quick access to a given report
    prc_time_table tt;  ///< the time-based table, used for quick access to the reports from a given second
    port_status_t timeout_implication;  ///< the implication for when a report times out
    prc_report_status_fn status_callback;  ///< the function to call with report status
    void *callback_context; ///< context to provide with the callback on report status
} packet_resp_canceller;


void init_packet_resp_canceller(packet_resp_canceller *self,int wait_secs,prc_report_status_fn status_callback,void *callback_context,port_status_t timeout_implication);
packet_resp_canceller *new_packet_resp_canceller(int wait_secs,prc_report_status_fn status_callback,void *callback_context,port_status_t timeout_implication);
void free_packet_resp_canceller(packet_resp_canceller *self);

void packet_resp_canceller_new_time(packet_resp_canceller *self,time_t time);

void packet_resp_canceller_add_report(packet_resp_canceller *self,spade_report *rpt);
void packet_resp_canceller_note_response(packet_resp_canceller *self,port_status_t implied_status,u32 sip,u16 sport,u32 dip,u16 dport,int portless);

void packet_resp_canceller_print_config_details(packet_resp_canceller *self,FILE *f,char *indent);

/*@}*/

#endif  /* ! PACKET_RESP_CANCELLER_H */


/* Id: packet_resp_canceller.h,v 1.7 2002/12/21 01:25:39 jim Exp $ */





typedef void (*netspade_exc_callback_t)(void *context,spade_report *rpt);
typedef void (*netspade_adj_callback_t)(void *context,char *id,char *mess,int using_corrrscore);

/// the type of xarg_type_t node value; either unsigned int or CIDR
typedef enum {XARG_TYPE_UINT,XARG_TYPE_CIDR} xarg_type_t;

/// an administatively excluded field value; this results from Xdips, Xsports, etc
typedef struct _xfeatval_link {
    features feat; ///< the field the restriction is on
    xarg_type_t type;  /// the type of the value for this restriction
    union {
        int i; /// the value to exclude, if type=XARG_TYPE_UINT
        struct {
            u32 netip; /// the network IP address in the CIDR
            u32 netmask;  /// the netmask in the CIDR
        } cidr; /// the network to exclude, if type=XARG_TYPE_CIDR
    } val;
    struct _xfeatval_link *next; /// the next element in a linked list of this type
} xfeatval_link;

struct _netspade;

/// encapsulates the state specific to a netspade detector
typedef struct _netspade_detector {
    /// the next netspade_detector in a linked list of them
    struct _netspade_detector *next;
    struct _netspade* parent;
    /// the id for this detector
    char *id;

    /// the type of detector this is; value is from spade_detect_types.h
    int detect_type;
    /// the packet conditions under which this detector will calculate the score and assess anomalousness
    event_condition_set scorecalc_conds;
    /// should this detector ignore packets with destination IPs that are broadcast IP addresses
    int exclude_broadcast_dip;
    /// if the detector is doing response waiting, this is the implication of a timeout
    port_status_t thresh_exc_port_impl;

    /// the packet conditions under which this detector will store an event in the probability table
    event_condition_set store_conds;
    /// the packet conditions for which this detector will check for a response in the open direction
    event_condition_set cancel_open_conds;
    /// the packet conditions for which this detector will check for a response in the closed direction
    event_condition_set cancel_closed_conds;
    /// the criterea required for a anomalous event to be reported (versus dropped)
    port_status_set_t port_report_criterea;
    
    /// the libspade entity which calculates anomaly scores
    score_calculator calculator;
    /// the libspade score manager
    score_mgr mgr;
    /// if non-NULL, the packet response canceller in use
    packet_resp_canceller *canceller;

    /// the shared state with the score manager
    spade_enviro enviro;
    /// the packet stats as of the last time the anomaly score was adjusted
    spade_pkt_stats last_adj_stats;
    
    /// a linked list of reports to exclude in this detector; suppliments netspade's global list
    xfeatval_link *rpt_exclude_list;

    /// \brief the detection type we report for this detector
    /// \note eventually, there may be more than one detection type possible from a given detector, so this will need to be generalized
    int report_detection_type;
    /// the string encoding the scope of this detector; used in building reports
    char *report_scope_str; 
} netspade_detector;

/// a link in a linked list of networks
typedef struct _ll_net {
    u32 netaddr; ///< the network address
    u32 netmask; ///< the network mask
    struct _ll_net *next;  ///< the next link in a linked list of networks
} ll_net;

/// a instance of netspade
typedef struct _netspade {
    /// the head of a linked list of detectors contained in this netspade
    netspade_detector *detectors;
    /// the tail of the linked list of detectors
    netspade_detector *detectors_tail;
    /// the packet conditions under which we need to pass an event to our event_recorder
    event_condition_set recorder_needed_conds;
    /// the packet conditions under which we need to do something besides storing a packet
    event_condition_set nonstore_conds;
    /// the the packet conditions that we actually need to calculate for any given packet
    event_condition_set conds_to_calc;

    ll_net *homelist_head; ///< the head a linked list of home networks
    ll_net *homelist_tail; ///< the tail in a linked list of home networks 

    char *checkpoint_file; ///< the name of the file to checkpoint to
    int checkpoint_freq; ///< the frequency (in recorded packet counts) with which to checkpoint

    /// records how many things have been recorded since the last checkpoint
    int records_since_checkpoint;
    /// the last packet time that we passed along to the enties that need it
    time_t last_time_forwarded;
    
    /// the callback to invoke when there is an anomalous event
    netspade_exc_callback_t exc_callback;
    /// the callback to invoke when the threshold has been adjusted in a detector, or NULL if none should be called
    netspade_adj_callback_t adj_callback;
    /// the netspade-user-defined context to provide with the callback
    void *callback_context;
    
    /// a pointer to a routine to call to make a copy of the "native" field of a spade_event, or NULL if none is needed
    /** this is used when a spade_event is being copied for storage in the response buffer (packet reponse canceller) */
    event_native_copier_t pkt_native_copier_callback;
    /// a pointer to a routine to call free a copy of the "native" field of a spade_event, or NULL if none is needed
    /** this is used when a copied spade_event is being freed when it is being removed from the packet reponse canceller */
    event_native_freer_t pkt_native_freer_callback;
    
    xfeatval_link *rpt_exclude_list; ///< a linked list of reports to exclude globally

    event_recorder recorder; ///< our event recorder

    u8 stats_to_print; ///< the statistics to print to the output log file
    char *outfile; ///< the name of the output log file

    int debug_level;  ///< the debug level we are at (bigger is higher/more verbose)
    spade_msg_fn msg_callback; ///< the function to call when we have a text message for the user
    
    int detector_id_nonce;  ///< the default detector id for the last detector that was set up
    unsigned long total_pkts; ///< the total number of packets passed to us
} netspade;


void init_netspade(netspade *self, spade_msg_fn msg_callback, int debug_level);
int init_netspade_from_statefile(netspade *self, char *statefile, spade_msg_fn msg_callback, int debug_level);
netspade *new_netspade(spade_msg_fn msg_callback, int debug_level);
netspade *new_netspade_from_statefile(char *statefile, spade_msg_fn msg_callback, int debug_level,int *succ);

void netspade_set_callbacks(netspade *self, void *context, netspade_exc_callback_t exc_callback, netspade_adj_callback_t adj_callback, event_native_copier_t pkt_native_copier_callback, event_native_freer_t pkt_native_freer_callback);
void netspade_set_checkpointing(netspade *self, char *checkpoint_file, int checkpoint_freq);
void netspade_set_homenet_from_str(netspade *self, char *homenet_str);
void netspade_set_output_stats(netspade *self, int stats_to_print);
void netspade_set_output_stats_from_str(netspade *self, char *str);
int netspade_set_output(netspade *self, char *file, int stats_to_print);
int netspade_set_output_file(netspade *self, char *file);
void netspade_add_rpt_excludes(netspade *self,char *xsips,char *xdips,char *xsports,char *xdports);

char *netspade_new_detector(netspade *self, char *str);

int netspade_set_detector_scaling(netspade *self, char *detectorid, int scale_freq, double scale_factor, double prune_threshold);
char *netspade_setup_detector_adapt_from_str(netspade *self, int adaptmode, char *str);
int netspade_setup_detector_adapt1(netspade *self, char *detectorid, int adapttarget, time_t period, float new_obs_weight, int by_count);
int netspade_setup_detector_adapt2(netspade *self, char *detectorid, double targetspec, double obsper, int NS, int NM, int NL);
int netspade_setup_detector_adapt3(netspade *self, char *detectorid, double targetspec, double obsper, int NO);
int netspade_setup_detector_advise(netspade *self, char *detectorid, int obs_size, int obs_secs);
char *netspade_setup_detector_advise_from_str(netspade *self, char *str);
int netspade_setup_detector_survey(netspade *self, char *detectorid, char *filename, float interval);
char *netspade_setup_detector_survey_from_str(netspade *self, char *str);

void netspade_new_pkt(netspade *self, spade_event *pkt);

void netspade_dump(netspade *self);
void netspade_cleanup(netspade *self);
void netspade_write_log(netspade *self);

char *netspade_detector_scope_str(netspade *self,char *id);

int netspade_print_detector_config_details(netspade *self,FILE *f,char *id);

void print_conds(event_condition_set conds);
void print_conds_line(event_condition_set conds);

/*@}*/

#endif // NETSPADE_H
/*********************************************************************
strtok.h, distributed as part of Spade v030125.1
Author: James Hoagland, Silicon Defense (hoagland@SiliconDefense.com)
copyright (c) 2002 by Silicon Defense (http://www.silicondefense.com/)
Released under GNU General Public License, see the COPYING file included
with the distribution or http://www.silicondefense.com/spice/ for details.

Please send complaints, kudos, and especially improvements and bugfixes to
hoagland@SiliconDefense.com.  As described in GNU General Public License, no
warranty is expressed for this program.
*********************************************************************/

#ifndef STRTOK_H
#define STRTOK_H

/*! \file strtok.h
 * \brief
 *  strtok.h is the header file for strtok.c.
 * \ingroup libspade_util
 */

/*! \addtogroup libspade_util
    @{
*/



int fill_args_space_sep(char *str,char *format,void *args[],spade_msg_fn msg_callback);
char *extract_str_arg_space_sep(char *str,char *argname);
int terminate_first_tok(char *str,char *sepchars,char **head,char *oldchar);

/*@}*/
#endif // STRTOK_H

/* Id: strtok.h,v 1.5 2002/12/19 22:37:10 jim Exp $ */
