import re
import sys
import time

import Parser
import util

class ParserRealSecure(Parser.Parser):

    eventEntryNames = [ 
        '6in4_Tunnel',
        'AIX_Pdnsd_Overflow',
        'AOLIM_AddExternalApp_Overflow',
        'AOLIM_File_Xfer',
        'AOLIM_GameRequest_Overflow',
        'AOLIM_Login',
        'AOLIM_Message',
        'AOLIM_Password_Change',
        'AOLIM_Trillian_Encrypt_Handshake',
        'AOL_Instant_Messenger_Overflow',
        'ARJ_Long_FileName_Detected',
        'ASP_Chunked_Overflow',
        'AUDIT_DNS_Version_Request',
        'Allaire_JRun_JSP_Execute',
        'Allaire_JRun_SSIFilter',
        'Allaire_JRun_Sample_Files',
        'Allaire_JRun_WebInf_DotSlash',
        'Allaire_JRun_WebInf_SlashSlash',
        'Alvgus_Request',
        'Alvgus_Response',
        'Alvgus_TCP_Request',
        'Alvgus_TCP_Response',
        'Amanda_TCP_Response',
        'AntiSniff_ARP_Test',
        'AntiSniff_DNS_Test',
        'AolAdmin_Response',
        'Ascend_Kill',
        'Ascend_Kill_II',
        'Asylum_Response',
        'Audit_TFTP_Get_Filename',
        'Avaya_Cajun_Default_SNMP',
        'BDDT_TCP_Response',
        'BGP_Illegal_Size',
        'BGP_New_Route',
        'BGP_Notify_Msg',
        'BGP_Open_Malformed',
        'BGP_Route_Unreachable',
        'BOOTP_Remote_Overflow',
        'BackConstruction_Response',
        'BackDoor_TCP_Response',
        'BackOrifice2K_TCP_Auth_Request',
        'BackOrifice2K_TCP_Auth_Response',
        'BackOrifice2K_TCP_Request',
        'BackOrifice2K_TCP_Response',
        'BackOrifice2K_UDP_Auth_Request',
        'BackOrifice2K_UDP_Auth_Response',
        'BackOrifice2K_UDP_Request',
        'BackOrifice2K_UDP_Response',
        'BackOrifice_Ping',
        'BackOrifice_Request',
        'BackOrifice_Response',
        'Backage_TCP_Request',
        'Backdoor2_Response',
        'Backdoor_CIA_Response',
        'Backdoor_CYN_Request',
        'Balistix_Request',
        'Balistix_Response',
        'BasicHell_TCP_Response',
        'Beast_TCP_Response',
        'BigGluck_Response',
        'Bigorna_TCP_Response',
        'BioNet_Response',
        'BitTorrent_Get_Request',
        'BitTorrent_Response',
        'Bla_Request',
        'BlackAngel_TCP_Response',
        'BladeRunner_TCP_Request',
        'BladeRunner_TCP_Response',
        'Blazer5_Response',
        'BloodFestEvolution_TCP_Response',
        'Boink',
        'Bonk',
        'Bootparam',
        'BrightStor_Uniagent_Overflow',
        'Bugs_Response',
        'Buschtrommel_TCP_Response',
        'Buttman_TCP_Request',
        'Buttman_TCP_Response',
        'CA_License_Server_Request_Bo',
        'CVS_Argumentx_Double_Free',
        'CVS_Auth_User_Failure',
        'CVS_Auth_User_Success',
        'CVS_Directory_Double_Free',
        'CVS_Notify_Underflow',
        'CVS_Request_Argument_Overflow',
        'CVS_Request_EntryLine_Overflow',
        'CVS_Request_Option_Overflow',
        'CVS_Request_Path_Overflow',
        'CVS_Request_Tag_Overflow',
        'Cafeini_TCP_Response',
        'Celine_TCP_Response',
        'Cerebus_Scanner',
        'Cero_TCP_Response',
        'Chargen_Denial_of_Service',
        'Chupacabra_Request',
        'CiscoSccp_Invalid_MessageID',
        'CiscoSccp_Message_Overflow',
        'CiscoSccp_Message_Underflow',
        'CiscoSccp_Open_Receive_Channel',
        'CiscoSccp_Start_Media_Transmission',
        'CiscoSccp_Stop_Media_Transmission',
        'Cisco_CR_DoS',
        'Cisco_Cable_Docsis_SNMP_Community',
        'Cisco_H323_Overflow',
        'Cisco_ILMI_SNMP_Community',
        'Cisco_IOS_IPV4_DoS',
        'Cisco_IOS_OSPF_BO',
        'Cisco_Ident',
        'Coma_Response',
        'ConnectionBackdoor_Response',
        'Content_Compound_File_Bad_Extension',
        'Content_Incorrect_Extension',
        'Content_RAR_Missing_Extension',
        'Corrupt_IP_Options',
        'CrackDown_TCP_Response',
        'CrazzyNet_Response',
        'CyberCop_Scanner_HTTP',
        'CyberCop_Scanner_ICMP',
        'CyberCop_Scanner_RPC',
        'CyberCop_Scanner_Radius',
        'CyberCop_Scanner_SMTP',
        'CyberCop_Scanner_TFTP',
        'Cyn_Request',
        'Cyn_Response',
        'Cyn_TCP_Request',
        'Cyn_TCP_Response',
        'DCOM_Large_Body_Extension',
        'DCOM_RemoteActivate',
        'DCOM_RemoteGetClassObject_DoS',
        'DCOM_Scada_Opc_Bind',
        'DCOM_Scada_Opc_Bo',
        'DCOM_SystemActivation',
        'DFchGrisch_TCP_Response',
        'DHCP_Ack',
        'DHCP_Discover',
        'DHCP_Domain_Metachar',
        'DHCP_Format_String_BO',
        'DHCP_Hostname_Overflow',
        'DHCP_Long_Discover_Message',
        'DHCP_Long_Hardware_Address',
        'DHCP_Minires_Format_Overflow',
        'DHCP_Request',
        'DHTML_IE_JavaScript_XSS',
        'DHTML_Object_Overflow',
        'DNS_Address_Length',
        'DNS_Antisniff_Overflow',
        'DNS_Authors_Request',
        'DNS_Bind_OPT_DoS',
        'DNS_Bind_SIG_Overflow',
        'DNS_Chaos_Request',
        'DNS_Crack_Success',
        'DNS_DNSSEC_Type_Mismatch',
        'DNS_Excessive_Requests',
        'DNS_Format_String',
        'DNS_Generic_Intel_Overflow',
        'DNS_HInfo',
        'DNS_Hostname_Overflow',
        'DNS_Hostname_Overflow_Verylong',
        'DNS_IDN_Query',
        'DNS_IQuery',
        'DNS_IQuery_bo',
        'DNS_Malformed',
        'DNS_Malformed_CompressedName',
        'DNS_NULL_Query',
        'DNS_NXT_Overflow',
        'DNS_NonInternet',
        'DNS_Poison',
        'DNS_Query_All',
        'DNS_Spoof_Failed',
        'DNS_Spoof_Success',
        'DNS_TSIG_Overflow',
        'DNS_Version_Request',
        'DNS_VirusScanTrojan',
        'DNS_WINS_DoS',
        'DNS_Windows_SMTP_Overflow',
        'DNS_Zero_Size_UDP',
        'DNS_Zone_Transfer',
        'DNS_Zonexfer_High',
        'DRat_TCP_Response',
        'DTr_TCP_Response',
        'Dameware_Detected',
        'Dameware_Obtain_Info',
        'Dameware_Spoof_Overflow',
        'Danton_TCP_Response',
        'DarkConnectionInside_TCP_Request',
        'DarkConnectionInside_TCP_Response',
        'DeepThroat_Response',
        'DeltaSource_Response',
        'Devil_Request',
        'DigitalRootBeer_TCP_Request',
        'DirectConnect_Connect',
        'Doly_Response',
        'DonaldDick_Response',
        'Dtspcd_Overflow',
        'Duddie_TCP_Request',
        'EMail_Generic_Intel_Overflow',
        'EPolicy_Orchestrator_Format_String',
        'EPolicy_Orchestrator_Vulnerable_Server',
        'Echo_Denial_of_Service',
        'Echo_Reply_Without_Request',
        'Edonkey_Connect',
        'Edonkey_Download',
        'Email_Almail_Overflow',
        'Email_Amavis_Exec',
        'Email_Auth_Failed',
        'Email_Auth_Overflow',
        'Email_BioNet',
        'Email_Command_Overflow',
        'Email_Data',
        'Email_Debug',
        'Email_Decode',
        'Email_Ehlo',
        'Email_Encap_Exch_Relay',
        'Email_Encap_Relay',
        'Email_Envid_Overflow',
        'Email_Error',
        'Email_ExchangeStore_DoS',
        'Email_Executable_Extension',
        'Email_Executable_Extension_Block',
        'Email_Expn',
        'Email_Expn_Overflow',
        'Email_False_Attachment',
        'Email_From',
        'Email_From_Overflow',
        'Email_HTML_File_URI',
        'Email_Helo_Overflow',
        'Email_IE_HRAlign_Overflow',
        'Email_IE_ObjectType_Overflow',
        'Email_Invalid_Command',
        'Email_Listserv_Overflow',
        'Email_Lotus_Domino',
        'Email_Mime_Filename',
        'Email_Mime_Filename_Blanks',
        'Email_Mime_Filename_Chars',
        'Email_Mime_Filename_Overflow',
        'Email_Mime_Name_Overflow',
        'Email_Mime_Null',
        'Email_Name_Overflow',
        'Email_Outlook_Date_Overflow',
        'Email_Outlook_URL_Spoof',
        'Email_Pipe',
        'Email_Potential_BO_Attachment',
        'Email_Qmail_Length',
        'Email_Qmail_Rcpt',
        'Email_Rcpt_TooManyQuotes',
        'Email_Recipient_Dot',
        'Email_Recipient_Overflow',
        'Email_Relay_Attempt',
        'Email_Relay_Spam',
        'Email_ReplyTo_Executable',
        'Email_Rpmmail_Alias',
        'Email_ServerID',
        'Email_SubSeven',
        'Email_Subject',
        'Email_To',
        'Email_To_Dot_Dot',
        'Email_Turn',
        'Email_UUDecode_Alias',
        'Email_VCF_Mozilla_Overflow',
        'Email_VCF_Overflow',
        'Email_Virus_Double_Extension',
        'Email_Virus_Double_Extension_Block',
        'Email_Virus_Iloveyou',
        'Email_Virus_Melissa',
        'Email_Virus_PICTURE_EXE',
        'Email_Virus_Papa',
        'Email_Virus_Suspicious_Zip',
        'Email_Virus_exploreZip',
        'Email_Virus_investigator',
        'Email_Vrfy',
        'Email_Vrfy_Overflow',
        'Email_WIZ',
        'Email_Xchg_Auth',
        'Email_Y3K',
        'EventHorizon_Request',
        'EventHorizon_TCP_Request',
        'EvilFTP_Response',
        'ExecutorController_TCP_Request',
        'Exploiter_TCP_Response',
        'FBackdoor_TCP_Response',
        'FIX_Logon',
        'FIX_Suspicious_MessageType',
        'FSP_Delete_File',
        'FSP_Detected',
        'FSP_Read_File',
        'FSP_Write_File',
        'FTP_AIX_Overflow',
        'FTP_Args_Overflow',
        'FTP_Auth_Failed',
        'FTP_Command_Overflow',
        'FTP_Commands_With_Binary',
        'FTP_Cwd_Overflow',
        'FTP_Cwd_Root',
        'FTP_Cwd_Tilde',
        'FTP_Cwd_concon',
        'FTP_Cwd_dotdot',
        'FTP_Cybercop_Scan',
        'FTP_Data_Overflow',
        'FTP_Delete_Very_Long',
        'FTP_Delete_dotdot',
        'FTP_FileName_Bdl',
        'FTP_File_Exec',
        'FTP_Filename',
        'FTP_Filename_Overflow',
        'FTP_Floppy_DoS',
        'FTP_Fname_Eftp2',
        'FTP_Fname_Lnk',
        'FTP_Format_String',
        'FTP_Generic_Intel_Overflow',
        'FTP_Get',
        'FTP_Glob_Expansion',
        'FTP_Glob_Implementation',
        'FTP_Glob_TildeBrace_Vulns',
        'FTP_Help_Overflow',
        'FTP_Invalid_Port_Cmd',
        'FTP_List_dotdot',
        'FTP_Login_Overflow',
        'FTP_Long_MKD',
        'FTP_Mdtm_Very_Long',
        'FTP_Mget_DotDot',
        'FTP_Mkd_Overflow',
        'FTP_Mkdir',
        'FTP_Mlst_Very_Long',
        'FTP_NLST_Overflow',
        'FTP_NetTerm_Dele_Overflow',
        'FTP_NetTerm_Dir_Overflow',
        'FTP_NetTerm_Ls_Overflow',
        'FTP_NetTerm_Mkd_Overflow',
        'FTP_NetTerm_Pass_Overflow',
        'FTP_NetTerm_Rmdir_Overflow',
        'FTP_Oracle_Pass_Overflow',
        'FTP_Pass',
        'FTP_Passive_Very_Long',
        'FTP_Password_Overflow',
        'FTP_Pasv',
        'FTP_Pasv_DOS',
        'FTP_Pipe',
        'FTP_Port',
        'FTP_Port_Bounce',
        'FTP_Port_Very_Long',
        'FTP_PrivilegedBounce',
        'FTP_PrivilegedPort',
        'FTP_ProFTPD',
        'FTP_ProFTPD_Translate_Overflow',
        'FTP_Put',
        'FTP_Restart_Very_Long',
        'FTP_Retr_Very_Long',
        'FTP_Retr_dotdot',
        'FTP_Rmd_Very_Long',
        'FTP_Rnfr_Very_Long',
        'FTP_Rnto_Very_Long',
        'FTP_Server_Identity',
        'FTP_Site_Chown_Overflow',
        'FTP_Site_Cmd',
        'FTP_Site_Cpwd',
        'FTP_Site_Exec',
        'FTP_Site_Exec_DotDot',
        'FTP_Site_Exec_Tar',
        'FTP_Site_Pswd',
        'FTP_Size_Very_Long',
        'FTP_Size_dotdot',
        'FTP_Stat_Very_Long',
        'FTP_Stor_Very_Long',
        'FTP_Syst',
        'FTP_Tar_Exec',
        'FTP_Unix_Password_File',
        'FTP_Unix_RemoteHost_File',
        'FTP_User',
        'FTP_User_Root',
        'FTP_Virus_Wm_Marker_A',
        'FTP_WSFTP_Allo_BO',
        'FTP_WUFTP_RealPath_BO',
        'FTP_Wildcard_DoS',
        'FTP_Windows_Drive_Path',
        'FTP_Windows_INI_File',
        'FTP_Windows_PWL_File',
        'FTP_Windows_SAM_File',
        'FTP_ZIPCHK_Meta',
        'FTP_ZIPCHK_Overflow',
        'FTP_dotdotdot',
        'FW1_Auth_As_Local',
        'FW1_Auth_Replay',
        'FW1_GetTopology',
        'FastTrack_Download',
        'FileNail_TCP_Request',
        'Finger_Backdoor',
        'Finger_Command',
        'Finger_Enumeration',
        'Finger_Filename',
        'Finger_Forwarding',
        'Finger_Forwarding_DOS',
        'Finger_Generic_Intel_Overflow',
        'Finger_List',
        'Finger_Overflow',
        'Finger_Overflow_RTM',
        'Finger_Root',
        'Finger_Scan',
        'Finger_Search',
        'Finger_User',
        'ForcedEntry_Response',
        'Fore_Response',
        'Fore_TCP_Request',
        'Fore_TCP_Response',
        'FragRoute_TCP_Chaff_PAWS',
        'Fraggle_Attack',
        'Fragment_Differential_Overlap',
        'Fragment_Differential_Size',
        'Fragment_Resources_Exhausted',
        'Freak88_Response',
        'Frenzy_Response',
        'GSpot_TCP_Request',
        'GSpot_TCP_Response',
        'GTP_C_APN_Corrupt',
        'GTP_C_Discovery',
        'GTP_C_Element_Overflow',
        'GTP_C_Element_Underflow',
        'GTP_C_Element_Unexpected',
        'GTP_C_Err_SystemFailure',
        'GTP_C_PPP_Login',
        'GTP_U_InfrastructureAddress',
        'GTP_U_Recursion',
        'GTP_U_StationToStation',
        'GateCrasher_Response',
        'Gauntlet_CyberDaemon_Overflow',
        'Gauntlet_ICMP_DoS',
        'GayOL_Request',
        'Ghost_TCP_Response',
        'Gift_TCP_Response',
        'GirlFriend_Response',
        'Glacier_Client_Request',
        'Glacier_Request',
        'Glacier_Response',
        'Glacier_Server_Request',
        'Gnutella_BearShare',
        'Gnutella_Connect',
        'Gnutella_Download',
        'Gnutella_LimeWire',
        'Gnutella_Shareaza',
        'Gnutella_Worm',
        'Guptachar_Response',
        'H225_Invalid_Field_DoS',
        'H225_Invalid_Length_DoS',
        'H225_Length_Bo',
        'H225_Signaling_Message_Order',
        'H225_Suspicious_Field_Length',
        'H323_Detected',
        'HPUX_RLPD_Overflow',
        'HP_OpenView_NNM_Overflow',
        'HP_OpenView_SNMP_Backdoor',
        'HSRP_Coup',
        'HSRP_Default_Password',
        'HSRP_Invalid_IPTTL',
        'HSRP_Resign',
        'HSRP_Suspicious_Priority',
        'HTML_AIM_Away_Overflow',
        'HTML_AIM_GetFile_Overflow',
        'HTML_CDF_XZone_Script',
        'HTML_Element_Filename_Traversal',
        'HTML_Element_Location_Traversal',
        'HTML_FTP_Command',
        'HTML_Hostname_Overflow',
        'HTML_IE_DragDrop_Executable',
        'HTML_IE_File_URI_Encode',
        'HTML_IE_Help_Save_File',
        'HTML_IE_Sysimage_Disclosure',
        'HTML_IE_Table_Spoof',
        'HTML_IRC_URL_Overflow',
        'HTML_Image_Source_Executable',
        'HTML_Outlook_Codebase_Execute',
        'HTML_Outlook_MailTo_Code_Execution',
        'HTML_URL_Homograph',
        'HTML_Win_Grpconv_Overflow',
        'HTTP_$DATA_Source_Disclosed',
        'HTTP_3com_AirConnect_EasySetup',
        'HTTP_3com_AirConnect_FilteringSetup',
        'HTTP_3com_AirConnect_FirmwareSetup',
        'HTTP_3com_AirConnect_ModemSetup',
        'HTTP_3com_AirConnect_RFSetup',
        'HTTP_3com_AirConnect_SNMPSetup',
        'HTTP_3com_AirConnect_SecuritySetup',
        'HTTP_3com_AirConnect_SpecialFunctions',
        'HTTP_3com_AirConnect_SystemSetup',
        'HTTP_ACCEPT_Overflow',
        'HTTP_ASP_Security_Bypass',
        'HTTP_ASP_Stateserver_Overflow',
        'HTTP_AWStats_ConfigDir_Exec',
        'HTTP_AWStats_LoadPlugin',
        'HTTP_AWStats_PluginMode_Exec',
        'HTTP_Abyss_Conf',
        'HTTP_Acrobat_ActiveX_Overflow',
        'HTTP_ActiveX',
        'HTTP_Adpassword_Text',
        'HTTP_Advancestack_BypassAuth',
        'HTTP_Aglimpse',
        'HTTP_AllaireJrun_ViewDirectory',
        'HTTP_AnswerBook2_Admin',
        'HTTP_AnswerBook2_DocServer',
        'HTTP_AnswerBook2_Execute',
        'HTTP_Answerbook_AddAdmin',
        'HTTP_Answerbook_Format_String',
        'HTTP_AnyForm',
        'HTTP_AnyForm_Post',
        'HTTP_Apache_Chunked_BO',
        'HTTP_Apache_Chunked_DoS',
        'HTTP_Apache_DOS',
        'HTTP_Apache_Jakarta_Format_String',
        'HTTP_Apache_Macros_dir',
        'HTTP_Apache_ModProxy_Bo',
        'HTTP_Apache_ModUserDir_Disclosure',
        'HTTP_Apache_PHP',
        'HTTP_Apache_ServerInfo',
        'HTTP_Apache_ServerStatus',
        'HTTP_Apache_SlashSlash',
        'HTTP_ApcRunner_ExecuteCode',
        'HTTP_Arcadia_Tradecli',
        'HTTP_AresGalaxy',
        'HTTP_Auctionweaver',
        'HTTP_AuthFilter_ISAPI_Overflow',
        'HTTP_Auth_Failed',
        'HTTP_Auth_TooLong',
        'HTTP_Authentication',
        'HTTP_Axis_Storpoint',
        'HTTP_BAT_Execute',
        'HTTP_BBN_survey',
        'HTTP_BadBlue_DOS',
        'HTTP_BadBlue_DirDisclosure',
        'HTTP_Bash_Shell_History',
        'HTTP_BigBrother_History',
        'HTTP_Bigsam',
        'HTTP_BioNet_ICQ_Pager',
        'HTTP_BrownOrifice',
        'HTTP_Bugbear_Backdoor',
        'HTTP_CGI_Fastgraf',
        'HTTP_CGI_Guestbook_Generic',
        'HTTP_CGI_Guestbook_Meta',
        'HTTP_CMD_Execute',
        'HTTP_Cachemgr',
        'HTTP_Calender_Admin',
        'HTTP_Campas',
        'HTTP_Carbo_Server',
        'HTTP_Carello',
        'HTTP_Cart32_ChangeAdminPassword',
        'HTTP_Cart32_ClientList',
        'HTTP_Cart32_Expdate',
        'HTTP_Cdomain',
        'HTTP_CgiCounterLong',
        'HTTP_Cgiproc_DoS',
        'HTTP_Cgiproc_File_Read',
        'HTTP_CheckLoginPhp',
        'HTTP_CheckPoint_FW1_FormatString',
        'HTTP_Chunked_Encoding_Overflow',
        'HTTP_Cisco_Aironet_Webconfig',
        'HTTP_Cisco_Catalyst_Exec',
        'HTTP_Cisco_IOS_Admin_Access',
        'HTTP_Cisco_IOS_DOS',
        'HTTP_Cisco_IOS_Query_DOS',
        'HTTP_Classifieds_Post',
        'HTTP_CobaltRAQ_OverflowCGI',
        'HTTP_CobaltRAQ_alert',
        'HTTP_CobaltRAQ_service',
        'HTTP_Code_Red',
        'HTTP_Code_Red_II',
        'HTTP_Code_Red_II_Plus',
        'HTTP_ColdFusion_Admin',
        'HTTP_ColdFusion_Cfcache',
        'HTTP_ColdFusion_Debug',
        'HTTP_ColdFusion_Email_ExampleApp',
        'HTTP_ColdFusion_Expr_Evaluator',
        'HTTP_ColdFusion_FileExists',
        'HTTP_ColdFusion_RDS_Null_Password',
        'HTTP_ColdFusion_RDS_Password',
        'HTTP_ColdFusion_SourceWindow',
        'HTTP_ColdFusion_SyntaxChecker_DOS',
        'HTTP_ColdFusion_ViewExample',
        'HTTP_ColdFusion_WebPublish_ExampleApp',
        'HTTP_Commerce',
        'HTTP_CompaqInsightManager_Taglist',
        'HTTP_Compaq_Insight_Cpqlogin_Overflow',
        'HTTP_Compaq_Insight_DoS',
        'HTTP_Compaq_Insight_Fileread',
        'HTTP_Compaq_Insight_FormatString',
        'HTTP_Connect',
        'HTTP_Connect_Proxy_Bypass_SMTP',
        'HTTP_Connection_Overflow',
        'HTTP_Content_Disposition_DotDot',
        'HTTP_Content_Disposition_Spoofing',
        'HTTP_Content_Type_HTA',
        'HTTP_Cookie',
        'HTTP_Count',
        'HTTP_Cross_Site_Scripting',
        'HTTP_CrystalReports_FileAccess_DoS',
        'HTTP_Csvform_Execute',
        'HTTP_DCForum_Admin_Access',
        'HTTP_DCForum_File_Upload',
        'HTTP_DCForum_Traversal',
        'HTTP_DCShop_info',
        'HTTP_Dansie_Backdoor',
        'HTTP_Dansie_Cart',
        'HTTP_Dansie_Infoleak',
        'HTTP_Dir_Manager_exe',
        'HTTP_Domino_Navigator',
        'HTTP_Domino_Web_Access',
        'HTTP_DosDevice',
        'HTTP_DotDot',
        'HTTP_DotDotDot',
        'HTTP_DragonFire',
        'HTTP_ECware_DOS',
        'HTTP_EDonkey',
        'HTTP_EPerl_isindex',
        'HTTP_EZMall_Malllogfile',
        'HTTP_EZShopper_Loadpage',
        'HTTP_EZShopper_Search',
        'HTTP_Empower',
        'HTTP_Eva_Forms_Bo',
        'HTTP_Eva_Forms_Dir',
        'HTTP_Favorites_Icon_Overflow',
        'HTTP_FaxSurvey',
        'HTTP_Field_With_Binary',
        'HTTP_Fields_With_Binary',
        'HTTP_FileTypeLnk',
        'HTTP_FileTypeUrl',
        'HTTP_FingerCgi',
        'HTTP_FormMail',
        'HTTP_FrontPage_Authors',
        'HTTP_FrontPage_DeviceName',
        'HTTP_FrontPage_PWD',
        'HTTP_Frontpage_Extensions_RAD_Overflow',
        'HTTP_Frontpage_Path',
        'HTTP_Frontpage_Publish',
        'HTTP_GET_ComputeSum',
        'HTTP_GET_CreateTable',
        'HTTP_GET_DotDot_Data',
        'HTTP_GET_Dotdotdot_Data',
        'HTTP_GET_Filename_mgtstate',
        'HTTP_GET_Filename_mysql',
        'HTTP_GET_Filename_pwl',
        'HTTP_GET_Filename_sam',
        'HTTP_GET_GroupBy',
        'HTTP_GET_SQL_OpenRowSet',
        'HTTP_GET_SQL_UnionAllSelect',
        'HTTP_GET_SQL_UnionSelect',
        'HTTP_GET_SQL_WaitForDelay',
        'HTTP_GET_Very_Long',
        'HTTP_GET_XP_Cmdshell',
        'HTTP_GETargscript',
        'HTTP_Gator_Installed',
        'HTTP_Generic_Intel_Overflow',
        'HTTP_Get',
        'HTTP_GetAccess_helpwin',
        'HTTP_GetAccess_login',
        'HTTP_GetArg',
        'HTTP_Get_Malformed',
        'HTTP_Glimpse',
        'HTTP_GoAhead_ASP_CodeRetrieval',
        'HTTP_GotoMyPCDOTCom_Connection',
        'HTTP_Groupwise_Path',
        'HTTP_Guestbook',
        'HTTP_HCP_Code_Execution',
        'HTTP_HOST_Overflow',
        'HTTP_HP_JetAdmin_ExecuteFile',
        'HTTP_HTDIG_htsearch',
        'HTTP_HTMLScript',
        'HTTP_HackATack_ICQ_Pager',
        'HTTP_Hassan_Execute',
        'HTTP_Head',
        'HTTP_HomeFree_Search',
        'HTTP_Host_Invalid_Char',
        'HTTP_HotMail_Email_Attachment',
        'HTTP_Htaccess',
        'HTTP_Htimage_Dir',
        'HTTP_Htimage_Exe',
        'HTTP_Htimage_Files',
        'HTTP_Html_In_Ref',
        'HTTP_ICQ_Pager',
        'HTTP_IERedir_Zone_Bypass',
        'HTTP_IE_ADODB_Stream_SaveToFile',
        'HTTP_IE_HRAlign_Overflow',
        'HTTP_IE_HTML_Embed_Overflow',
        'HTTP_IE_Help_Overflow',
        'HTTP_IE_IFrame_BO',
        'HTTP_IE_InstallEngineCtl_Overflow',
        'HTTP_IE_MHT_Overflow',
        'HTTP_IE_Script_HRAlign_Overflow',
        'HTTP_IE_Security_Zone_Bypass',
        'HTTP_IE_ShellExecute',
        'HTTP_IE_Status_Spoof',
        'HTTP_IE_Style_Heap_BO',
        'HTTP_IExplorer_Cache_Exec',
        'HTTP_IExplorer_Command_Exec',
        'HTTP_IIS3_Asp_Dot',
        'HTTP_IISExAir_DOS',
        'HTTP_IISHTR_Overflow',
        'HTTP_IIS_ASP_Chunked_Overflow',
        'HTTP_IIS_Bdir_Htr',
        'HTTP_IIS_CmdAsp',
        'HTTP_IIS_Csimple',
        'HTTP_IIS_Device_Asp',
        'HTTP_IIS_Double_Eval_Evasion',
        'HTTP_IIS_FPSE_Debug_Bo',
        'HTTP_IIS_HTR_Chunked_Overflow',
        'HTTP_IIS_Hex_Evasion',
        'HTTP_IIS_Host_DoS',
        'HTTP_IIS_ISAPI_Filter_Error_DoS',
        'HTTP_IIS_ISAPI_Nsiislog_Overflow',
        'HTTP_IIS_ISAPI_Printer_Overflow',
        'HTTP_IIS_IndexSearch',
        'HTTP_IIS_Index_Server_Overflow',
        'HTTP_IIS_Invalid_File_Parsing',
        'HTTP_IIS_MSSQL_XML_Script',
        'HTTP_IIS_MSSQL_xml',
        'HTTP_IIS_Many_Hosts',
        'HTTP_IIS_Media_Services',
        'HTTP_IIS_Obtain_Code',
        'HTTP_IIS_Percent_Evasion',
        'HTTP_IIS_Showcode',
        'HTTP_IIS_Track',
        'HTTP_IIS_Trailing_Incomplete_Unicode',
        'HTTP_IIS_Trailing_Slash',
        'HTTP_IIS_UTF8_Evasion',
        'HTTP_IIS_Unicode_Encoding',
        'HTTP_IIS_Unicode_Wide_Encoding',
        'HTTP_IIS_dumpvariables',
        'HTTP_IIS_htr_isapi',
        'HTTP_IIS_ssi',
        'HTTP_IRunbook_FileDisclosure',
        'HTTP_ISA_Rules_Engine_Bypass',
        'HTTP_Icecast_mp3',
        'HTTP_Imagemap_Exe',
        'HTTP_IndexServer_IDQ',
        'HTTP_IndexServer_SQLQHit',
        'HTTP_IndexServer_Source_Disclosure',
        'HTTP_IndexServer_Webhits',
        'HTTP_Info2WWW',
        'HTTP_InterScan_VirusWall_Overflow',
        'HTTP_InterscanViruswall_ChgCfg',
        'HTTP_InterscanViruswall_RegGo',
        'HTTP_InterscanViruswall_Smtpscan',
        'HTTP_JJ_Overflow',
        'HTTP_JRun_Double_Slash',
        'HTTP_JRun_IIS_Overflow',
        'HTTP_JRun_ISAPI_Host',
        'HTTP_JSP_SourceRead',
        'HTTP_Java',
        'HTTP_Java_Class',
        'HTTP_Java_Webadmin_BBS',
        'HTTP_Kazaa',
        'HTTP_KnowledgeBuilder_CodeExecution',
        'HTTP_LastlinesCgi_CmdExecute',
        'HTTP_LastlinesCgi_Traversal',
        'HTTP_Listrec_Execute',
        'HTTP_Listserv_Waexe',
        'HTTP_Location_Bypass_Restriction',
        'HTTP_Lock_Method_DOS',
        'HTTP_Login_Known_User',
        'HTTP_Long_Header_Name',
        'HTTP_Lotus_Domino_Dir_Traversal',
        'HTTP_Lotus_Domino_Obtain_Info',
        'HTTP_MDAC_Access',
        'HTTP_MDAC_Get',
        'HTTP_MDAC_RDS_Overflow',
        'HTTP_MSCS_ProfileSvc_Overflow',
        'HTTP_MSIS_Script',
        'HTTP_MSRadio_Overflow',
        'HTTP_MSWord_Doc_Overflow',
        'HTTP_MachineInfo',
        'HTTP_Mailsite_Wconsole',
        'HTTP_Mambo_Phpsessid',
        'HTTP_Manolito',
        'HTTP_MaxDB_websql_BO',
        'HTTP_Mdaemon_Form2Raw_BO',
        'HTTP_MediaWiki_Executable_Upload',
        'HTTP_Media_Player_Code_Execution',
        'HTTP_Media_Player_setItemInfo_CodeExec',
        'HTTP_Mozilla_Nonascii_URL_BO',
        'HTTP_MyDoom_DoS',
        'HTTP_NT8.3_Filename',
        'HTTP_Napster_Installed',
        'HTTP_Negotiate_SSP_ASN1_Overflow',
        'HTTP_Netscape_List_Directories',
        'HTTP_Netscape_Method_Overflow',
        'HTTP_Netscape_PageServices',
        'HTTP_Netscape_Revlog',
        'HTTP_Netscape_SpaceView',
        'HTTP_Netscape_URI_Overflow',
        'HTTP_Netscape_WPauth',
        'HTTP_Netscape_rend',
        'HTTP_NetwareWebserver_Traversal',
        'HTTP_Netware_DirList',
        'HTTP_Nfuse_Path',
        'HTTP_Nfuse_Script',
        'HTTP_Nimda_Worm',
        'HTTP_Novell_Convert',
        'HTTP_Novell_Files',
        'HTTP_NphTestCgi',
        'HTTP_OWC_Installer_Overflow',
        'HTTP_OWC_Vulnerable_Client',
        'HTTP_Object_Type_Overflow',
        'HTTP_OfficeScan_Auth_Bypass',
        'HTTP_Officescan_Config',
        'HTTP_Omnihttp',
        'HTTP_OracleAdmin_Web_Interface',
        'HTTP_OracleAdmin_help_overflow',
        'HTTP_OracleApp_AggreSpy',
        'HTTP_OracleApp_DMSDump',
        'HTTP_OracleApp_Info',
        'HTTP_OracleApp_Spy',
        'HTTP_OracleApp_XSQL',
        'HTTP_OracleApp_demo_info',
        'HTTP_OracleApp_demo_sendmail',
        'HTTP_OracleApp_dms0',
        'HTTP_OracleApp_oprocmgr_service',
        'HTTP_OracleApp_oprocmgr_status',
        'HTTP_OracleApp_soap',
        'HTTP_Oracle_Appserver_Overflow',
        'HTTP_Oracle_Appserver_ViewInfo',
        'HTTP_Oracle_Appserver_rwcgi60',
        'HTTP_Oracle_Batchfile',
        'HTTP_Oracle_CharConversion',
        'HTTP_Oracle_WebCache_Overflow',
        'HTTP_Oracle_iSQL_Login_Overflow',
        'HTTP_OrderLog',
        'HTTP_OrderLogDat',
        'HTTP_OrderLogV12Dat',
        'HTTP_Outlook_Codebase_Execute',
        'HTTP_PDGSoft_Changepw',
        'HTTP_PHF_CommandExec',
        'HTTP_PHPMyAdmin_EvalExecute',
        'HTTP_PHPMyAdmin_SqlPhp',
        'HTTP_PHPMyAdmin_Sql_Include',
        'HTTP_PHPNuke_Admin_Access',
        'HTTP_PHPNuke_Admin_Overwrite',
        'HTTP_PHPNuke_Index_File',
        'HTTP_PHPNuke_ModulesPhp_DOS',
        'HTTP_PHPNuke_Prefix_Admin',
        'HTTP_PHPNuke_ReadFiles',
        'HTTP_PHPNuke_URL_Redirect',
        'HTTP_PHP_Addslashes_ViewFiles',
        'HTTP_PHP_Includedir',
        'HTTP_PHP_LoadPrefs',
        'HTTP_PHP_Magic_Quote_Dir',
        'HTTP_PHP_Memchr_BO',
        'HTTP_PHP_Overflow',
        'HTTP_PHP_Phorum',
        'HTTP_PHP_ReadFile',
        'HTTP_PHP_Realpath_Bypass',
        'HTTP_POST_ComputeSum',
        'HTTP_POST_Content_Length_Negative',
        'HTTP_POST_CreateTable',
        'HTTP_POST_Filename_WinIni',
        'HTTP_POST_Filename_passwd',
        'HTTP_POST_Filename_sam',
        'HTTP_POST_GroupBy',
        'HTTP_POST_PeopleSoft_IClientServlet',
        'HTTP_POST_PeopleSoft_Traversal',
        'HTTP_POST_PeopleSoft_XXE',
        'HTTP_POST_SQL_OpenRowSet',
        'HTTP_POST_SQL_UnionAllSelect',
        'HTTP_POST_SQL_UnionSelect',
        'HTTP_POST_SQL_WaitForDelay',
        'HTTP_POST_Script',
        'HTTP_POST_XP_Cmdshell',
        'HTTP_POST_dotdot_data',
        'HTTP_POST_dotdotdot_data',
        'HTTP_POST_repeated_char',
        'HTTP_POUND_LogMsg_Bo',
        'HTTP_Passwd_Txt',
        'HTTP_Perl_Example_Code',
        'HTTP_Pfdispaly_Access',
        'HTTP_Pfdispaly_Execute',
        'HTTP_Pfdispaly_Read',
        'HTTP_PhatBot_AgoBot',
        'HTTP_PhpBB_Viewtopic_Highlight',
        'HTTP_PhpRocket_Traversal',
        'HTTP_Php_Include_Bypass',
        'HTTP_Pi3Web_TstIsapiDll',
        'HTTP_PlanetIntra_Overflow',
        'HTTP_Polycom_Reveal_Password',
        'HTTP_Post',
        'HTTP_PostQueryCgi',
        'HTTP_Post_Field',
        'HTTP_Post_Filename_pwl',
        'HTTP_Propfind_Very_Long',
        'HTTP_PsaPhp_RevealSource',
        'HTTP_Put',
        'HTTP_QuikStore',
        'HTTP_RPC_Connect',
        'HTTP_Realserver_Noarg',
        'HTTP_RegEcho',
        'HTTP_RingZero',
        'HTTP_Road_Search',
        'HTTP_RobotsTxt',
        'HTTP_RpcNLog',
        'HTTP_SCO_View_Source',
        'HTTP_SGI_Handler',
        'HTTP_SGI_Infosrch',
        'HTTP_SGI_Webdist',
        'HTTP_SGI_Wrap',
        'HTTP_Sambar_cgitest',
        'HTTP_ScanMail_Auth_Bypass',
        'HTTP_Script_Bypass',
        'HTTP_SendTempPl',
        'HTTP_Server_ID',
        'HTTP_SharePoint_Auth_Bypass',
        'HTTP_SharePoint_Script',
        'HTTP_Shell_Handler_Executable',
        'HTTP_Shells_Bash',
        'HTTP_Shells_C',
        'HTTP_Shells_Ksh',
        'HTTP_Shells_Perl',
        'HTTP_Shells_Perl_Exe',
        'HTTP_Shells_Rksh',
        'HTTP_Shells_Sh',
        'HTTP_Shells_Tcsh',
        'HTTP_ShockWave',
        'HTTP_Shopper_Conf',
        'HTTP_SiteCsc_Access',
        'HTTP_SiteServer_Post_Dir_Traversal',
        'HTTP_Skype',
        'HTTP_Skype_Callto_Overflow',
        'HTTP_SoftCart_Order_Info',
        'HTTP_SoftCart_StoreMgrPw',
        'HTTP_Sojourn',
        'HTTP_Spyki_PhpInclude_Worm',
        'HTTP_Squid_ACL_Bypass',
        'HTTP_Squid_Cache_Poisoning',
        'HTTP_Squid_NTLM_Password_Bo',
        'HTTP_StoreCGI',
        'HTTP_SubSevenCGI',
        'HTTP_SubSeven_ICQ_Pager',
        'HTTP_SubscribeMe_Setup',
        'HTTP_Sunone_Viewlog',
        'HTTP_Swat_AuthBo',
        'HTTP_TektronixPrinter',
        'HTTP_Tilde',
        'HTTP_Tomcat_Nulllist',
        'HTTP_Trace',
        'HTTP_Translate_F_SourceRead',
        'HTTP_TrendVCS_Auth_Bypass',
        'HTTP_Twiki_Search_CmdExec',
        'HTTP_UBB_Posting_Topic',
        'HTTP_URL_BackslashDotDot',
        'HTTP_URL_Bad_Hex_Code',
        'HTTP_URL_FrontPage_Service_Password',
        'HTTP_URL_Frontpage_Ext_Backdoor',
        'HTTP_URL_IIS_Password_Change_Attempt',
        'HTTP_URL_JspPath',
        'HTTP_URL_MLog',
        'HTTP_URL_MS_ADC_Samples',
        'HTTP_URL_Many_Slashes',
        'HTTP_URL_MyLog',
        'HTTP_URL_Name_Very_Long',
        'HTTP_URL_NewDsnExe',
        'HTTP_URL_Repeated_Dot',
        'HTTP_URL_Requests_Too_Often',
        'HTTP_URL_Snork_Bat',
        'HTTP_URL_WWWBoard_Script',
        'HTTP_URL_dotpath',
        'HTTP_URL_repeated_char',
        'HTTP_URLscan',
        'HTTP_Unify_UploadServelet',
        'HTTP_Unix_Passwords',
        'HTTP_UserAgent_Too_Long',
        'HTTP_User_Agent',
        'HTTP_Verity_Search',
        'HTTP_ViaVideo_Overflow',
        'HTTP_Viewsrc_fileread',
        'HTTP_Vulnerable_Client',
        'HTTP_W3Who_Bo',
        'HTTP_WWWThreads_Admin',
        'HTTP_Wayboard_Fileview',
        'HTTP_WeaknessCGIScanner',
        'HTTP_WebDAV_Long_Rqst_BO',
        'HTTP_WebDAV_Long_Rqst_DOS',
        'HTTP_WebDAV_XML_Attribute_DoS',
        'HTTP_WebFinger',
        'HTTP_WebLogic_FileServlet_ShowCode',
        'HTTP_WebLogic_FileSourceRead',
        'HTTP_WebLogic_PluginBO',
        'HTTP_WebLogic_fileread',
        'HTTP_WebManager_Httpsave_overflow',
        'HTTP_WebPALS',
        'HTTP_WebSite_Sample',
        'HTTP_WebSite_Uploader',
        'HTTP_WebSphere_Host',
        'HTTP_WebSphere_reveal_path',
        'HTTP_WebStore_Misconfig',
        'HTTP_WebcartCgi_Execute',
        'HTTP_Webevent',
        'HTTP_Webinf_Dot_FileRetrieval',
        'HTTP_Weblogic_SSIServlet_ShowCode',
        'HTTP_Webmin_Traversal',
        'HTTP_WebpageCgi_ViewInfo',
        'HTTP_Webplus',
        'HTTP_Websendmail',
        'HTTP_Webspeed_Admin',
        'HTTP_WhiskerScan',
        'HTTP_Whois',
        'HTTP_WikkiTikkiTavi_Remote_Template',
        'HTTP_WinMX',
        'HTTP_Win_Repair',
        'HTTP_WindMail_FileRead',
        'HTTP_Windows_Executable',
        'HTTP_WmvDownloader_B',
        'HTTP_XStat',
        'HTTP_Y3K_ICQ_Pager',
        'HTTP_YaBB',
        'HTTP_Yahoo_Email_Attachment',
        'HTTP_Yahoo_YAutoDLL_BO',
        'HTTP_YouSendIt_Email_Attachment',
        'HTTP_Zero_Hex_Code',
        'HTTP_ZmlCgi',
        'HTTP_cookieOverflow',
        'HTTP_formmailpl',
        'HTTP_iHTML_Merchant_File_Access',
        'HTTP_iPlanet_Search_BO',
        'HTTP_iPlanet_Search_ViewFiles',
        'HTTP_lotsOfURLs',
        'HTTP_mailnews_post',
        'HTTP_phpBB2',
        'HTTP_repeated_character',
        'HTTP_rguestexe',
        'HTTP_testcgi',
        'HTTP_testcgitcl',
        'HTTP_webgais',
        'HTTP_wguestexe',
        'HVL_RAT_Response',
        'HackATack_Ping',
        'HackATack_TCP_Response',
        'HackATack_UDP_Response',
        'Hacker_Defender',
        'HackersParadise_Response',
        'Helix_RealServer_Overflow',
        'Helix_Server_DoS',
        'HellzAddiction_TCP_Response',
        'Hidepak_Rootkit_Login',
        'Hidesource_Rootkit_Login',
        'HostControl_Response',
        'ICMP6_BSD_Dos',
        'ICMP_Flood',
        'ICMP_Modem_DoS',
        'ICMP_PingTunnel_Detected',
        'ICMP_Protocol_Unreachable_TCP',
        'ICMP_Redirect',
        'ICMP_Source_Quench',
        'ICMP_Subnet_Mask_Request',
        'ICMP_TCP_MTU_DoS',
        'ICMP_Timestamp_Request',
        'ICMP_Unreachable_Storm',
        'ICMPv6_Malformed_Option_Segment',
        'ICQ_PAM_Parser_Overflow',
        'ICQ_Witty_Worm',
        'IDENT_Generic_Intel_Overflow',
        'IIS_FTP_Session_Status_DoS',
        'IIS_Reveal_Address',
        'IMAP4_Very_Long_Command',
        'IMAP_Auth_Failed',
        'IMAP_Generic_Intel_Overflow',
        'IMAP_Imail_Overflow',
        'IMAP_Login',
        'IMAP_Long_Authname',
        'IMAP_Long_Login',
        'IMAP_Long_Mbox',
        'IMAP_Long_Parm',
        'IMAP_Long_Password',
        'IMAP_Password',
        'IMAP_Too_Many_Connects',
        'IMAP_WU_Imapd_BO',
        'IM_Activity',
        'IM_File_Xfer_Double_Extension',
        'IPV6_Bad_Fragment_Chain',
        'IP_Duplicate',
        'IP_Flushot',
        'IP_Fragment_Empty',
        'IP_Hdr_Options_Zero_Length',
        'IP_Invalid_Option',
        'IP_Microfragment',
        'IP_OShare',
        'IP_PingOfDeath_Jolt2',
        'IP_Ping_Of_Death_Jolt',
        'IP_SS_Ping',
        'IP_SourceRoute',
        'IP_Timestamp_Not_Aligned',
        'IP_Tunnel_Bad_Version',
        'IP_Unknown_Protocol',
        'IPv6_Native',
        'IPv6_Teredo',
        'IRC_Automated_Client',
        'IRC_DCC_Chat',
        'IRC_DCC_Request',
        'IRC_DCC_Send',
        'IRC_Daemon_Overflow',
        'IRC_Generic_Intel_Overflow',
        'IRC_Generic_Trojan',
        'IRC_Global_Threat',
        'IRC_Join',
        'IRC_Join_Attempt',
        'IRC_Litmus',
        'IRC_Msg',
        'IRC_Nick',
        'IRC_Notice',
        'IRC_Notice_Msg',
        'IRC_PrettyPark_Worm',
        'IRC_SubSeven_Notification',
        'IRC_TrinityV3_Notification',
        'IRC_Trinity_Notification',
        'IRC_Vscan',
        'IRC_Y3K_Notification',
        'IRDP_Gateway_Spoof',
        'ISAKMP_ASN1_Overflow',
        'ISAKMP_Certificate_Request_Overflow',
        'ISAKMP_Delete_SPI_DOS',
        'ISAKMP_Payload_Overflow',
        'ISAKMP_SPI_Overflow',
        'ISAKMP_Vendor_Id',
        'ISS_Ident',
        'ISS_Scan_7_0_ICMP',
        'ISS_Scan_FTP',
        'ISS_Scan_ICMP',
        'ISS_Scan_UDP',
        'ISS_SunRPC',
        'ISS_Wireless_Scanner_DHCP',
        'ITunes_OpenShare_Detected',
        'Ident_Error',
        'Ident_Flood',
        'Ident_Invalid_Response',
        'Ident_Newline',
        'Ident_Overflow',
        'Ident_Scan',
        'Ident_Suspicious_ID',
        'Ident_User',
        'Ident_Version',
        'Illusion_Backdoor_Response',
        'Image_ANI_Header_Overflow',
        'Image_ANI_RateNumber_DoS',
        'Image_BMP_Header_Overflow',
        'Image_Cursor_Malformed',
        'Image_EMF_Integer_Overflow',
        'Image_GIF_CompressionError',
        'Image_GIF_MSNMessenger_Execute_Code',
        'Image_GIF_Netscape_Extension_BO',
        'Image_ICON_Malformed',
        'Image_JPEG_Malformed',
        'Image_JPEG_Tag_Overflow',
        'Image_PNG_Height_Bo',
        'Image_PNG_Malformed',
        'Image_PNG_tRNS_BO',
        'Image_RIFF_Malformed',
        'InCommand_TCP_Request',
        'Infector_FTP_Response',
        'Infector_Response',
        'Infector_TCP_Request',
        'Infector_TCP_Response',
        'InfraTrojan_Request',
        'Insane_Network_Backdoor',
        'InstallShield_Silent_Installer',
        'Interscan_Viruswall_Httpsave',
        'Intruzzo_TCP_Response',
        'Kerberos_ASN1_Overflow',
        'Kerberos_User_Snarf',
        'KidTerror_Response',
        'Kika_TCP_Request',
        'Kika_UDP_Request',
        'Known_Rootkit_Login',
        'Konik_TCP_Response',
        'Kuang2Virus_Response',
        'LDAP_Distinguished_Name_Overflow',
        'LDAP_Filter_Overflow',
        'LDAP_IMail_Overflow',
        'LDAP_Message',
        'LDAP_Null_Field',
        'LDAP_Request_Failed',
        'LDAP_Server_ASN1_Overflow',
        'LHA_File_Path_Overflow',
        'LPRng_Format_String',
        'LanMan_Share_Enum',
        'LanMan_Share_Enum_Sweep',
        'Land_Attack',
        'Last2000_TCP_Request',
        'Latinus_TCP_Response',
        'LeGuardien_TCP_Request',
        'LeakTest_Request',
        'Leszcz_TCP_Response',
        'Linux_v2_Rootkit_Login',
        'Linux_v4_Rootkit_Login',
        'Loki',
        'M2Trojan_TCP_Response',
        'MDB_MSJet_File_Overflow',
        'MGCP_LongField',
        'MGCP_Long_Endpoint',
        'MGCP_Long_Tid',
        'MSMessenger_FileXfer',
        'MSMessenger_Login',
        'MSMessenger_Message',
        'MSRPC_Anon_User_Enumeration',
        'MSRPC_Domaindump',
        'MSRPC_Dump',
        'MSRPC_LLS_Bo',
        'MSRPC_LSASS_Bo',
        'MSRPC_LSASS_Request_Detected',
        'MSRPC_LSA_Crash',
        'MSRPC_LSA_DoS',
        'MSRPC_Locator_BO',
        'MSRPC_Lookupnames',
        'MSRPC_MSMQ_Overflow',
        'MSRPC_Malformed_DOS',
        'MSRPC_Message_Que_Heap_BO',
        'MSRPC_NetDDE_Bo',
        'MSRPC_Pipe_SAMR',
        'MSRPC_Pipe_SAMR_Failed',
        'MSRPC_Popup_Message',
        'MSRPC_RFPoison_Attack',
        'MSRPC_Race_Heap_Overflow',
        'MSRPC_Registry_Key',
        'MSRPC_Registry_Permissions',
        'MSRPC_Registry_PolicySecrets',
        'MSRPC_Registry_Read_MS_Index_Server',
        'MSRPC_Registry_Read_Sql_Execpass',
        'MSRPC_Registry_Request_DoS',
        'MSRPC_Registry_SAM',
        'MSRPC_Registry_Write',
        'MSRPC_Registry_Write_IMail_Admin',
        'MSRPC_Registry_Write_RASMan',
        'MSRPC_Registry_Write_RDS',
        'MSRPC_Registry_Write_Run',
        'MSRPC_Registry_Write_aedebug',
        'MSRPC_RemoteActivate_Bo',
        'MSRPC_RemoteActivate_Path_BO',
        'MSRPC_Security_Id_Lookup',
        'MSRPC_Share_Dump',
        'MSRPC_Share_Enum',
        'MSRPC_Share_Enum_Sweep',
        'MSRPC_Suspicious_UUID',
        'MSRPC_Svcctl_Remote_Control',
        'MSRPC_Svcctl_Remote_Query',
        'MSRPC_User_Enum',
        'MSRPC_WksSvc_Mgmnt_Bo',
        'MS_Compressed_Folders_Overflow',
        'Malformed_Packet_Storm',
        'Manolito_Download',
        'Mantis_TCP_Response',
        'Massaker_TCP_Response',
        'MastersParadise98_Response',
        'MavericksMatrix_Response',
        'MeetTheLamer_TCP_Request',
        'Michal_TCP_Response',
        'MicroSpy_TCP_Response',
        'Microsoft_Windows_Shell_Banner',
        'Millenium_Response',
        'Mneah_TCP_Request',
        'Mneah_TCP_Response',
        'MoSucker_TCP_Response',
        'MoonPie_FTP_Response',
        'MoonPie_TCP_Response',
        'MountdExport',
        'Mstream_Master',
        'Mstream_Zombie_Request',
        'Mstream_Zombie_Response',
        'MyDoom_Backdoor',
        'MyDoom_SYNFlood',
        'MySQL_Change_User_Auth_Bypass',
        'MySQL_Check_Scramble_Auth_Bypass',
        'MySQL_User_Password_Overflow',
        'NCX_Backdoor_Response',
        'NNTP_Cancel_Overflow',
        'NNTP_Control',
        'NNTP_From_Overflow',
        'NNTP_Group',
        'NNTP_MesgID_Overflow',
        'NNTP_Password',
        'NNTP_Pipe',
        'NNTP_Subject',
        'NNTP_User',
        'NNTP_Xchg_Auth',
        'NOSecure_TCP_Request',
        'NTP_Readvar_Overflow',
        'NTP_SNTP_Malformed',
        'NTP_Time',
        'Nachi_Ping_Protection',
        'Nachi_Ping_Sweep',
        'Napster_Client_Update',
        'Napster_Create_Account',
        'Napster_Download',
        'Napster_Login',
        'Napster_Login_Info',
        'Napster_Private_Msg',
        'Napster_Public_Msg',
        'Napster_Search',
        'Napster_Sharing',
        'Nessus_Scanner',
        'Nestea',
        'NetAdministrator_TCP_Request',
        'NetAdministrator_TCP_Response',
        'NetBus_Pro_Request',
        'NetBus_Response',
        'NetController_TCP_Request',
        'NetController_TCP_Response',
        'NetDevil_TCP_Response',
        'NetMeeting_Directory_Traversal',
        'NetRaider_TCP_Request',
        'NetRaider_TCP_Response',
        'NetSphere_Client',
        'NetSphere_FTP_Response',
        'NetSphere_Response',
        'NetSpyV12_Response',
        'NetSpy_Response',
        'NetTaxi_TCP_Response',
        'Netbios_Name_Scan',
        'Netbios_NetDDE_Bo',
        'Netbios_Session_Granted',
        'Netbios_Session_Rejected',
        'Netbios_Session_Request',
        'NetworkTerrorist_TCP_Response',
        'Network_Normal',
        'Network_Quiet',
        'NewSilencer_TCP_Response',
        'NewTear',
        'Nmap_OS_Fingerprint',
        'Nmap_Ping',
        'NokNok_FTP_Response',
        'NokNok_TCP_Request',
        'NokNok_TCP_Response',
        'OSPF_Database_Desc',
        'OSPF_Database_Desc_Multicast',
        'OSPF_Hello',
        'OSPF_Hello_Multicast',
        'OSPF_Invalid_Packet_Length',
        'OSPF_Link_State_Ack',
        'OSPF_Link_State_Ack_Multicast',
        'OSPF_Link_State_Update',
        'OSPF_Link_State_Update_Multicast',
        'OSPF_Lnk_State_Req',
        'OSPF_Lnk_State_Req_Multicast',
        'OSPF_Null_Authentication',
        'OSPF_Short_Header',
        'OSPF_Unknown_Msg_Type',
        'Oblivion_TCP_Response',
        'Ohhara_Rootkit_Login',
        'Optix_Pro_Backdoor_Request',
        'Oracle_Admin_SdoCodeSize_Bo',
        'Oracle_DropSiteInstant_Bo',
        'Oracle_GenReplSupport_Bo',
        'Oracle_InstantiateOffline_Bo',
        'Oracle_InstantiateOnline_Bo',
        'Oracle_InterConvFuncts_Bo',
        'Oracle_Listener_Bo',
        'Oracle_Listener_Debug_DoS',
        'Oracle_MD2_SdoCodeSize_Bo',
        'Oracle_MD2_ValidateGeom_Bo',
        'Oracle_ToChar_Bo',
        'Oracle_Username_Bo',
        'Overnet_Search',
        'P2P_Activity',
        'PAM_Internal_Error',
        'PDE_Renew_Host',
        'PDE_Unauthenticated_Host',
        'POP3_Auth_Failed',
        'POP_APOP_Name_Overflow',
        'POP_Command_Overflow',
        'POP_Filename',
        'POP_Filename_Chars',
        'POP_Filename_Overflow',
        'POP_Filename_Space_Overflow',
        'POP_Fold_Overflow',
        'POP_Fuseware_Overflow',
        'POP_Generic_Intel_Overflow',
        'POP_List_Overflow',
        'POP_Login_Overflow',
        'POP_Password',
        'POP_Password_Overflow',
        'POP_QPopAuth_Overflow',
        'POP_QPopUser_Overflow',
        'POP_Response_Client_Bo',
        'POP_Retr_DoS',
        'POP_Server_Identity',
        'POP_SilentRunner_Pass_Overflow',
        'POP_SilentRunner_User_Overflow',
        'POP_Too_Many_Connects',
        'POP_User',
        'POP_User_Root',
        'POP_YPOPs_Overflow',
        'PPTP_Badmagic',
        'Packet_Capturing_Remote',
        'Packet_Capturing_Tool',
        'PcnfsdExec',
        'PhaseZero_Response',
        'Phatbot_P2P',
        'Phoenix_TCP_Response',
        'PingOfDeath',
        'Ping_Flood',
        'Ping_Sweep',
        'PlugAndPlay_BO',
        'PlugAndPlay_DoS',
        'PmapMnt',
        'PmapProxy',
        'PortalOfDoom_Response',
        'Portmaster_Reboot',
        'Progenic_Response',
        'Project_Next_Response',
        'Prosiak_Response',
        'Protos_H225_Attack_Tool',
        'PsExec_Installed',
        'PsExec_Service_Accessed',
        'Psychward_TCP_Response',
        'QAZ_Backdoor_Email',
        'QAZ_Backdoor_Request',
        'QAZ_Backdoor_Response',
        'Quake3Arena_Vulnerable_Client',
        'Quake3Arena_Vulnerable_Server',
        'Quake_Backdoor_Request',
        'Queso_Scan',
        'QwertosRat_TCP_Response',
        'R0xR4t_TCP_Response',
        'RAT_Content_Advisor_Overflow',
        'RDP_Login',
        'RIP_Add',
        'RIP_Expire',
        'RIP_Metric_Change',
        'RPC_Admind_Auth',
        'RPC_Amd_Overflow',
        'RPC_Amd_Pid',
        'RPC_Amd_Version',
        'RPC_AutoFS_Shell',
        'RPC_Automountd_Overflow',
        'RPC_CALLIT_Request',
        'RPC_Cachefsd_Overflow',
        'RPC_CallIt_Unknown',
        'RPC_Cmsd_Overflow',
        'RPC_Corrupt_Credentials',
        'RPC_GETPORT_Request',
        'RPC_Generic_Intel_Overflow',
        'RPC_GetPort',
        'RPC_KCMS_File_Retrieval',
        'RPC_Large_Fragmented',
        'RPC_Mount_Overflow',
        'RPC_Mountd_Mnt',
        'RPC_NFS_FHSize_Overflow',
        'RPC_NFS_Guess',
        'RPC_NFS_Mknod',
        'RPC_NFS_Over_LOCKD',
        'RPC_NFS_UID_Zero',
        'RPC_NIS_Overflow',
        'RPC_PCNFSD_Backdoor',
        'RPC_PcnfsdShell',
        'RPC_Pcnfsd_Overflow',
        'RPC_Portmap_Dump',
        'RPC_Portmap_Getport',
        'RPC_Portmap_High_Port',
        'RPC_Portmap_Set',
        'RPC_Portmap_Set_Spoof',
        'RPC_Portmap_Unset',
        'RPC_Portmap_Unset_Spoof',
        'RPC_Probe',
        'RPC_SGI_Fam_Access',
        'RPC_Sadmind_Ping',
        'RPC_Statd_Automount_Exec',
        'RPC_Statd_DotDot',
        'RPC_Statd_Format_Attack',
        'RPC_Statd_MyName_Format_String',
        'RPC_Statd_Overflow',
        'RPC_Suspicious_Host_Credentials',
        'RPC_TTDBServer_Format_String',
        'RPC_Tooltalkd_Overflow',
        'RPC_Xfsmd_Execute',
        'RPC_YPPasswd_Overflow',
        'RPC_Ypbind_DirClimb',
        'RPC_Ypupdate_Exec',
        'RPC_snmpXdmid_Overflow',
        'RSync_Vulnerable_Server',
        'Radius_AccessPacket_DoS',
        'Radius_AcctStatusType_Dos',
        'Radius_Generic_Intel_Overflow',
        'Radius_Password_Buffer_Overflow',
        'Radius_User_Buffer_Overflow',
        'RealSecure_Kill',
        'RedHat_LPD_Print_Control',
        'RemoteHack_TCP_Response',
        'RemoteRevise_TCP_Request',
        'RemoteStorm_Response',
        'RemoteWatch_Response',
        'RemoteWindowsShutdown_Response',
        'Rendezvous_Detected',
        'Retina_Scanner',
        'Rexd',
        'Rexec',
        'Rlogin_Auth_Failed',
        'Rlogin_Excessive_Tabs',
        'Rlogin_Froot',
        'Rlogin_Known_User',
        'Rlogin_Login',
        'Rlogin_Name_Very_Long',
        'Rlogin_Password_Overflow',
        'Rlogin_TERM_Very_Long',
        'Rlogin_Trusted_Login',
        'Rsh_Session',
        'Ruler_TCP_Response',
        'RuxTick_Request',
        'Rwhod_Malformed',
        'Rwhod_Overflow',
        'SIIT_Tunnel',
        'SIP_Blank_Header_Value',
        'SIP_Contact_Overflow',
        'SIP_Content_Length_Mismatch',
        'SIP_Large_Content_Length',
        'SIP_Large_Max_Forwards',
        'SIP_Long_Header_Name',
        'SIP_Long_Header_Value',
        'SIP_Long_Request_URI',
        'SMB_ASN1_Overflow',
        'SMB_Abel_Backdoor',
        'SMB_Admin_Sneak',
        'SMB_Auth_Failed',
        'SMB_Autoexec_bat_Access',
        'SMB_Client_Cleartext_Password',
        'SMB_Empty_Password',
        'SMB_Empty_Password_Failed',
        'SMB_Exception',
        'SMB_Executable_Access',
        'SMB_Filename',
        'SMB_Filename_Overflow',
        'SMB_Find_Files2_BO',
        'SMB_Guessable_Password',
        'SMB_IO_Printer_Share',
        'SMB_LSA_Connect',
        'SMB_Malformed',
        'SMB_NT_Transact_Bo',
        'SMB_Nimda_Worm',
        'SMB_Password_File',
        'SMB_Password_Overflow',
        'SMB_Pwl_Filetype',
        'SMB_RFParalyze',
        'SMB_Real_Path',
        'SMB_Sam_File',
        'SMB_Samba_SecurityDescriptor_Bo',
        'SMB_Samba_Transaction2_bo',
        'SMB_Service_Sweep',
        'SMB_Shutdown_Request',
        'SMB_Startup_File_Access',
        'SMB_System32_FileWritten',
        'SMB_Transact_Bo',
        'SMB_Unicode_Filename_Overflow',
        'SMB_Win_ini_file',
        'SMB_Winreg_File',
        'SMS_Remote_Service_DoS',
        'SMTP_BDAT_Large_Chunk',
        'SMTP_Etrn_Overflow',
        'SMTP_Exchange_Verb_DoS',
        'SMTP_Expn_Metachar',
        'SMTP_Loopback_Bounce',
        'SMTP_Malformed_BDAT',
        'SMTP_Malformed_Rcpt_Cmd',
        'SMTP_Nondeliverable_Notification',
        'SMTP_ParseAddr_Overflow',
        'SMTP_Probe_Root',
        'SMTP_Relay_Not_Allowed',
        'SMTP_Routing_Integer_Overflow',
        'SMTP_Routing_Overflow',
        'SMTP_Sendmail_Header_Parse_Overflow',
        'SMTP_Too_Many_Connects',
        'SMTP_User_Unknown',
        'SMTP_Virus_Recipient',
        'SMTP_YPOPs_Overflow',
        'SNMP_3COM_Communities',
        'SNMP_Activity',
        'SNMP_AuthTraps_Disable',
        'SNMP_Bad_ErrorIndex',
        'SNMP_Bad_ErrorStatus',
        'SNMP_Bad_Generic_Trap',
        'SNMP_Bad_RequestId',
        'SNMP_Bad_Specific_Trap',
        'SNMP_Bad_Trap_AgentAddr',
        'SNMP_Bad_Variable_Type',
        'SNMP_CiscoMgmt_Ping',
        'SNMP_Cisco_Passwd',
        'SNMP_Cisco_VACM_MIB',
        'SNMP_Cisco_Zero_Size_DOS',
        'SNMP_Community',
        'SNMP_Community_Format_String',
        'SNMP_Community_Long',
        'SNMP_Community_Underflow',
        'SNMP_Corrupt_Traffic',
        'SNMP_Counter64',
        'SNMP_Crack',
        'SNMP_Default_Backdoor',
        'SNMP_Delete_WINS',
        'SNMP_Dialup_Hangup',
        'SNMP_Dialup_Phone_Number',
        'SNMP_Dialup_Username',
        'SNMP_Discovery_Broadcast',
        'SNMP_Echo_Bounce',
        'SNMP_FormatString',
        'SNMP_HP_ProCurve',
        'SNMP_HP_Route_Metachar',
        'SNMP_Indefinite_Length',
        'SNMP_Int_Overflow',
        'SNMP_Int_Underflow',
        'SNMP_InvalidTag_Community',
        'SNMP_InvalidTag_Enterprise',
        'SNMP_InvalidTag_ErrorIndex',
        'SNMP_InvalidTag_ErrorStatus',
        'SNMP_InvalidTag_GenericTrap',
        'SNMP_InvalidTag_OID',
        'SNMP_InvalidTag_PDU',
        'SNMP_InvalidTag_Packet',
        'SNMP_InvalidTag_RequestID',
        'SNMP_InvalidTag_SpecificTrap',
        'SNMP_InvalidTag_Trap_Timestamp',
        'SNMP_InvalidTag_VarBind',
        'SNMP_InvalidTag_VarBindList',
        'SNMP_InvalidTag_Version',
        'SNMP_Lanman_Enum',
        'SNMP_Length_Underflow',
        'SNMP_Long_Field_Length',
        'SNMP_Long_OID',
        'SNMP_NonZeroLength_NullType',
        'SNMP_NullInCommunity',
        'SNMP_NullInString',
        'SNMP_OID_Overflow',
        'SNMP_OID_Underflow',
        'SNMP_PDU_Underflow',
        'SNMP_Packet_Underflow',
        'SNMP_RMON_Collections',
        'SNMP_Set',
        'SNMP_Show_LMUsers',
        'SNMP_Sun_Backdoor',
        'SNMP_Suspicious_Get',
        'SNMP_Suspicious_Set',
        'SNMP_Suspicious_Version_Size',
        'SNMP_SysContact',
        'SNMP_Sysname_Overflow',
        'SNMP_TFTP_Retrieval',
        'SNMP_TooManyVariables',
        'SNMP_Trap_Address_Underflow',
        'SNMP_Trap_Enterprise_Invalid',
        'SNMP_Trap_Enterprise_Underflow',
        'SNMP_Trap_Timestamp_Underflow',
        'SNMP_UInt_Overflow',
        'SNMP_UInt_Underflow',
        'SNMP_Value_Underflow',
        'SNMP_VarBindList_Underflow',
        'SNMP_VarBind_Underflow',
        'SNMP_Version_Underflow',
        'SNMP_ifTable',
        'SNMP_ipForwarding',
        'SNMP_ipRouteTable',
        'SNMP_trapAuthFailure',
        'SOAP_AccessDenied',
        'SOAP_Envelope_Overflow',
        'SOAP_Message_Body',
        'SOCKS4_Connect',
        'SOCKS5_Connect',
        'SOCKS5_Username_BO',
        'SOCKS_Bounce',
        'SOCKS_Over_SOCKS',
        'SQL_Auth_Failed',
        'SQL_Jet_ODS_Overflow',
        'SQL_Jet_Query_Overflow',
        'SQL_LogExplorer_Overflow',
        'SQL_Login',
        'SQL_MultipleXP_Overflow',
        'SQL_OpenRowSet_Overflow',
        'SQL_PasswordArray_Overflow',
        'SQL_PreAuth_Overflow',
        'SQL_PwdEncrypt_Overflow',
        'SQL_SSRP_DoS',
        'SQL_SSRP_Enum_Request',
        'SQL_SSRP_Enum_Response',
        'SQL_SSRP_HeapBo',
        'SQL_SSRP_MDAC_Client_Overflow',
        'SQL_SSRP_Malformed_Enum_Response',
        'SQL_SSRP_Slammer_Worm',
        'SQL_SSRP_StackBo',
        'SQL_Spida_Worm',
        'SQL_XP_WeakPermissions',
        'SSH_Banner_Length',
        'SSH_Brute_Force',
        'SSH_ChallengeReponse_Bo',
        'SSH_Deattack_IO',
        'SSH_Version',
        'SSH_Vulnerable_OpenSSH',
        'SSL2_Master_Key_Overflow',
        'SSL_ASN1_Overflow',
        'SSL_Challenge_Length_Overflow',
        'SSL_Hello_Msg_DoS',
        'SSL_Malformed_Certificate',
        'SSL_PCT1_Overflow',
        'SSL_Unsolicited_Certificate',
        'STUN_KPhone_DoS',
        'STUN_Message',
        'STUN_Message_Attribute',
        'SYNFlood',
        'Sadmind_Amslverify_Overflow',
        'Saihyousen_Attack',
        'Sasser_Propagation',
        'Satan_FTP',
        'Satan_UDP',
        'SatansBackdoor_Response',
        'Scarab_TCP_Response',
        'Schwindler_Response',
        'SecretService_Response',
        'SelSvcH',
        'SensorStatistics',
        'SensorStatistics_Cumulative',
        'ServeMe_Response',
        'Slapper_Worm',
        'Smurf_Attack',
        'SnidX2_Response',
        'SniperNet_Response',
        'Snoop_GetQuota_Overflow',
        'Snork_Attack',
        'Snort_Stream4_HeapBo',
        'SocketsDeTroie_Request',
        'Socks_Auth_Failed',
        'Solaris_D13HH_Rootkit_Login',
        'Solaris_LPD_Overflow',
        'Soulseek_Login_Detected',
        'Spyware_PH_180SearchAssistant',
        'Spyware_PH_ABetterInternet',
        'Spyware_PH_BargainBuddy',
        'Spyware_PH_BrilliantDigital',
        'Spyware_PH_BroadcastPC',
        'Spyware_PH_DownloadAcceleratorPlus',
        'Spyware_PH_DownloadWare',
        'Spyware_PH_EliteBar',
        'Spyware_PH_ExactSearchBar',
        'Spyware_PH_EzulaTopText',
        'Spyware_PH_GAIN',
        'Spyware_PH_GameSpyArcade',
        'Spyware_PH_HotBar',
        'Spyware_PH_IEPlugin',
        'Spyware_PH_KeenValue',
        'Spyware_PH_MarketScore',
        'Spyware_PH_MessengerPlus',
        'Spyware_PH_MoeMoneyMaker',
        'Spyware_PH_MySearchBar',
        'Spyware_PH_MyWebSearch',
        'Spyware_PH_QuickSearchBar',
        'Spyware_PH_ShopAtHomeSelect',
        'Spyware_PH_WeatherBug',
        'Spyware_PH_WhenUSearch',
        'Spyware_PH_WildTangent',
        'Stacheldraht_Agent',
        'Stacheldraht_Handler',
        'StealthSpy_Response',
        'Stream_DoS',
        'Stumbler_ChangeAddress',
        'Stumbler_Connect',
        'Stumbler_Probe',
        'SubSeven_Pass_Response',
        'SubSeven_Response',
        'SubSeven_Scan_Response',
        'Sun_SNMP_Backdoor',
        'Sunrpc_rwall_Message',
        'Sunrpc_rwall_Message_Overflow',
        'Sunrpc_rwall_Msg_Format_String',
        'Suspicious_ActiveX_Installer',
        'Swift_Request',
        'Symantec_NBNS_Bo',
        'Symantec_NBNS_Heap_Corruption',
        'Symantec_UPX_BO',
        'SynDrop',
        'Synthesized_Host_Attack_Flood',
        'Synthesized_Network_Attack_Flood',
        'Syphillis_Response',
        'Syphillis_Scan_Request',
        'Syphillis_Scan_Response',
        'Syslog_Cisco_Zero_Size_DoS',
        'TCC_Response',
        'TCP_ACK_Ping',
        'TCP_Connection_Flood',
        'TCP_Dabber_Sweep',
        'TCP_Data_Changed',
        'TCP_FIN_Scan',
        'TCP_Fin_OutOfRange',
        'TCP_Flag_SynUrg',
        'TCP_Frag_RFProwl',
        'TCP_Fragmented_Header',
        'TCP_Hijacking_Tool',
        'TCP_Invalid_SACK',
        'TCP_Multiple_Syns',
        'TCP_Network_Scan',
        'TCP_Network_Sweep',
        'TCP_Null_Scan',
        'TCP_OS_Fingerprint',
        'TCP_Port_Scan',
        'TCP_Probe_BitTorrent',
        'TCP_Probe_DNS',
        'TCP_Probe_Finger',
        'TCP_Probe_Ftp',
        'TCP_Probe_Gnutella',
        'TCP_Probe_HTTP',
        'TCP_Probe_IRC',
        'TCP_Probe_Ident',
        'TCP_Probe_Imap4',
        'TCP_Probe_LinuxConf',
        'TCP_Probe_Lpr',
        'TCP_Probe_MSRPC',
        'TCP_Probe_NNTP',
        'TCP_Probe_NetBIOS',
        'TCP_Probe_Netbus',
        'TCP_Probe_Other',
        'TCP_Probe_POP3',
        'TCP_Probe_PPTP',
        'TCP_Probe_Proxy',
        'TCP_Probe_Rlogin',
        'TCP_Probe_SMTP',
        'TCP_Probe_SQL',
        'TCP_Probe_Socks',
        'TCP_Probe_Sub7',
        'TCP_Probe_SunRPC',
        'TCP_Probe_T0rn',
        'TCP_Probe_Telnet',
        'TCP_Probe_Trojan',
        'TCP_Probe_XWindows',
        'TCP_Seq_OutOfRange',
        'TCP_Service_Sweep',
        'TCP_Short_Header',
        'TCP_Small_Segment_Size',
        'TCP_Trojan_Numeric_Response',
        'TCP_Urg_OutOfRange',
        'TCP_Within_Window_DoS',
        'TCP_Xmas_Scan',
        'TCP_Zero_Length_Option',
        'TFN2K_ICMP_Command',
        'TFN2K_TCP_Command',
        'TFN2K_UDP_Command',
        'TFN_Daemon',
        'TFTP_Alcatel_Update',
        'TFTP_Echo_Bounce',
        'TFTP_Exe_Transfer',
        'TFTP_File_Not_Found',
        'TFTP_Filename_Overflow',
        'TFTP_Filename_VeryLong',
        'TFTP_Get',
        'TFTP_Hosts_File',
        'TFTP_Image_Transfer',
        'TFTP_MSBlaster_Attempt',
        'TFTP_MSBlaster_Worm',
        'TFTP_Nachi_Worm',
        'TFTP_Nimda_Query',
        'TFTP_Nimda_Response',
        'TFTP_PWL_File',
        'TFTP_Passwd_File',
        'TFTP_Put',
        'TFTP_Sam_File',
        'TFTP_Traversal',
        'TFTP_WinIni_File',
        'Talk_Flash',
        'Talk_Request',
        'TearDrop',
        'TearDrop2',
        'TeleCommando_Response',
        'Telnet_Abuse',
        'Telnet_Auth_Failed',
        'Telnet_Auth_Kerb4',
        'Telnet_Auth_Kerb5',
        'Telnet_Auth_Loki',
        'Telnet_Auth_Null',
        'Telnet_Auth_Rsa',
        'Telnet_Auth_Spx',
        'Telnet_Auth_User',
        'Telnet_Auth_User_Overflow',
        'Telnet_Bad_IFS',
        'Telnet_Bad_Term',
        'Telnet_Bad_Xdisplay',
        'Telnet_Client_SLC_Long_Reply',
        'Telnet_Env_All',
        'Telnet_Env_Overflow',
        'Telnet_Env_Termcap',
        'Telnet_Environ_FTA',
        'Telnet_Excessive_Tabs',
        'Telnet_Inline_Demo_Capslock',
        'Telnet_Known_User',
        'Telnet_Linker_Bug',
        'Telnet_Login',
        'Telnet_Login_Overflow',
        'Telnet_NTLM_Tickle',
        'Telnet_Password_Overflow',
        'Telnet_PolycomDoS',
        'Telnet_Polycom_Blank_Password',
        'Telnet_Resolve_Host_Conf',
        'Telnet_Solaris_Forced_Login',
        'Telnet_Term_Type_Overflow',
        'Telnet_Terminal_Type',
        'Telnet_Too_Many_AYTs',
        'Telnet_Xdisplay',
        'Telnet_Xdisplay_Overflow',
        'TheFlu_TCP_Request',
        'TheRevenger_Request',
        'TheRevenger_Response',
        'TheThing_Response',
        'Tini_Response',
        'Tivoli_LCF_Default_Password',
        'Tivoli_LCF_File_Read',
        'TotalEclypse_Response',
        'Trace_Route',
        'Trace_Route_UDP',
        'TransScout_Request',
        'Trin00_Daemon_Request',
        'Trin00_Daemon_Response',
        'Trinoo_Master_Request',
        'Trinoo_Master_Response',
        'TrojanCow_TCP_Request',
        'Tron_TCP_Request',
        'Tron_TCP_Response',
        'Truva_Response',
        'Twinge_Attack',
        'UDP_Bomb',
        'UDP_Network_Scan',
        'UDP_Network_Sweep',
        'UDP_Port_Loopback',
        'UDP_Port_Scan',
        'UDP_Probe_CharGen',
        'UDP_Probe_DNS',
        'UDP_Probe_Echo',
        'UDP_Probe_MSDNS',
        'UDP_Probe_MSRPC',
        'UDP_Probe_NFS',
        'UDP_Probe_NFS_Lockd',
        'UDP_Probe_Norton_AV',
        'UDP_Probe_Other',
        'UDP_Probe_Qotd',
        'UDP_Probe_SNMP',
        'UDP_Probe_TFTP',
        'UDP_Probe_Trojan',
        'UDP_Quake3_Exploit',
        'UDP_Service_Sweep',
        'UDP_Short_Header',
        'UDP_Squid_WCCP_Cachelist_DOS',
        'UDP_Syslogd_BO',
        'UPX_Packed_Executable',
        'Ultors_Response',
        'Unexplained_Backdoor_Response',
        'VNC_Detected',
        'VNC_HTTP_Get_Overflow',
        'VNC_Login_Failed',
        'VNC_Login_Failed_User',
        'VNC_NoAuthentication',
        'VNC_RFBConnFailed_Overflow',
        'VagrNocker_TCP_Response',
        'VagrNoker_Response',
        'Vampire_TCP_Response',
        'Veritas_BackupExec_BO',
        'VoodooDoll_TCP_Request',
        'WINS_DelAssoc_Bo',
        'WINS_DelAssoc_DoS',
        'WINS_PointerHijack',
        'WarTrojan_TCP_Request',
        'Wh00t_Rootkit_Login',
        'WhatsUpPing',
        'Win2K_DomainController_DOS',
        'WinCrash_TCP_Response',
        'WinMX_Download',
        'WinNuke_Attack',
        'WinTrin00_Daemon_Request',
        'WinTrin00_Daemon_Response',
        'Win_IGMP_DOS',
        'Win_IP_Src_Route',
        'Win_MessengerPopup_Bo',
        'WindowsMite_TCP_Response',
        'Windows_Access_Error',
        'Windows_NNTP_Overflow',
        'Windows_Null_Session',
        'Windows_Printing_Service_DOS',
        'WoW23_TCP_Response',
        'XDMCP_Overflow',
        'XFS_Invalid_Opcode',
        'XLog_TCP_Response',
        'X_ZToo_Backdoor_Response',
        'Xanadu_TCP_Request',
        'Xanadu_TCP_Response',
        'Xanadu_UDP_Request',
        'Xanadu_UDP_Response',
        'Y3K_Ping',
        'Y3K_TCP_Response',
        'Y3K_UDP_Response',
        'YahooMSG_AddView',
        'YahooMSG_File_Transfer',
        'YahooMSG_Filename_Overflow',
        'YahooMSG_Login',
        'YahooMSG_Message',
        'YahooMSG_Message_Chat',
        'YahooMSG_PeertoPeer',
        'YahooMSG_URL_Handler_Overflow',
        'YahooMSG_UserID_Overflow',
        'Zincite_Backdoor',
        'Zincite_Backdoor_FileExecute',
        'connection0000',
        'connection0001',
        'connection0002',
        'connection0003',
        'connection0004',
        'connection0005',
        'connection0006',
        'connection0007',
        'pcAnywhere_Login',
        'pcAnywhere_Login_Failed',
        'pcAnywhere_Ping',
        'pcAnywhere_Probe',
        'sscan_Scan',    ]

    def process(self):

        if self.plugin["source"] == 'snmp':
            while 1: self.__processSyslog()
            
        else:
            util.debug (__name__,  "log type " + self.plugin["source"] +\
                        " unknown for RealSecure...", '!!', 'RED')
            sys.exit()


    def __processSyslog(self):
        
        util.debug ('ParserRealSecure', 'plugin started (syslog)...', '--')

        start_time = time.time()

        pattern = 'ISS-MIB::eventEntryName\d*\.\"\"\s+=\s+STRING:\s+' +\
            '(\S+)\s+' +\
            'ISS-MIB::eventEntryTime\d*\.\"\"\s+=\s+STRING:\s+' +\
            '(\d+-\d+-\d+\s+\d+:\d+:\d+)\s+' +\
            'ISS-MIB::eventEntryProtocol\d*\.\"\"\s+=\s+STRING:\s+' +\
            '(\S+)\s+\(\d+\)\s+' +\
            'ISS-MIB::eventEntrySourceIpAddress\d*\.\"\"\s+=\s+STRING:\s+' +\
            '(\S+)\s+' +\
            'ISS-MIB::eventEntryDestinationIpAddress\d*\.\"\"\s+=\s+STRING:\s+' +\
            '(\S+)\s+' +\
            'ISS-MIB::eventEntryIcmpType\d*\.\"\"\s+=\s+STRING:\s+' +\
            '(\S+)\s+' +\
            'ISS-MIB::eventEntryIcmpCode\d*\.\"\"\s+=\s+STRING:\s+' +\
            '(\S+)\s+' +\
            'ISS-MIB::eventEntrySourcePort\d*\.\"\"\s+=\s+STRING:\s*' +\
            '(\S+)\s+' +\
            'ISS-MIB::eventEntryDestinationPort\d*\.\"\"\s+=\s+STRING:\s*' +\
            '(\S+)\s+'

        location = self.plugin["location"]
        try:
            fd = open(location, 'r')
        except IOError, e:
            util.debug(__name__, e, '!!', 'RED')
            sys.exit()
        priority = 1
            
        # Move to the end of file
        fd.seek(0, 2)
           
        while 1:
            
            if self.plugin["enable"] == 'no':

                # plugin disabled, wait for enabled
                util.debug (__name__, 'plugin disabled', '**', 'YELLOW')
                while self.plugin["enable"] == 'no':
                    time.sleep(1)
                    
                # lets parse again
                util.debug (__name__, 'plugin enabled', '**', 'GREEN')
                fd.seek(0, 2)

            where = fd.tell()
            line = fd.readline()
            if not line: # EOF reached
                time.sleep(1)
                fd.seek(where)

                # restart plugin every hour
                if self.agent.plugin_restart_enable:
                    current_time = time.time()
                    if start_time + \
                       self.agent.plugin_restart_interval < current_time:
                        util.debug(__name__, 
                                   "Restarting plugin..", '->', 'YELLOW')
                        fd.close()
                        start_time = current_time
                        return None

            else:

                # first one line with the priority
                if   line.__contains__('lowpriorityevent'):
                    priority = 1
                elif line.__contains__('mediumpriorityevent'):
                    priority = 2
                elif line.__contains__('highpriorityevent'):
                    priority = 3
                    
                # afterwards the real secure alert
                else:

                    result = re.findall(str(pattern), line)
                    try: 
                        (name, date, protocol, src_ip, dst_ip, icmp_type,
                        icmp_code, src_port, dst_port) = result[0]
                        sid = ParserRealSecure.eventEntryNames.index(name) + 1

                        self.agent.sendEvent  (type = 'detector',
                                         date       = date,
                                         sensor     = self.plugin["sensor"],
                                         interface  = self.plugin["interface"],
                                         plugin_id  = self.plugin["id"],
                                         plugin_sid = sid,
                                         priority   = priority,
                                         protocol   = protocol,
                                         src_ip     = src_ip,
                                         src_port   = src_port,
                                         dst_ip     = dst_ip,
                                         dst_port   = dst_port,
                                         log        = "")

                    except IndexError: 
                        pass
                    except ValueError:
                        util.debug (__name__, 
                                    "eventEntryName %s unknown" % name, 
                                    "**", "RED");
        fd.close()


